'Copyright 2006-2012 ConnectCode Pte Ltd. All Rights Reserved.
'This source code is protected by International Copyright Laws. You are only allowed to 
'use the functions under the terms and license of ConnectCode Barcode Software.

'Version 2.0 (w8.5) - 10 June 2012

'======================================================================================================
'Code39
'======================================================================================================
   
Public Function Encode_Code39(ByVal data As String, Optional ByVal chk As Integer = 1) As String

      
    Dim result As String
    Dim cd As String
    Dim filtereddata As String
            
    filtereddata = filterInput_Code39(data)
    filteredlength = Len(filtereddata)

    If chk = 1 Then
    
        If filteredlength > 254 Then
        
            filtereddata = left$(filtereddata, 254)
            
        End If
        cd = generateCheckDigit_Code39(filtereddata)
    
    Else
    
        If filteredlength > 255 Then
            
            filtereddata = left$(filtereddata, 255)
            
        End If
    
    End If

    result = "*" + filtereddata + cd + "*"
    Encode_Code39 = result

End Function

Public Function getCode39Value(ByVal inputchar As Integer) As Integer

    Dim CODE39MAP() As Variant
    CODE39MAP = Array(Asc("0"), Asc("1"), Asc("2"), Asc("3"), Asc("4"), Asc("5"), Asc("6"), Asc("7"), Asc("8"), Asc("9"), Asc("A"), Asc("B"), Asc("C"), Asc("D"), Asc("E"), Asc("F"), Asc("G"), Asc("H"), Asc("I"), Asc("J"), Asc("K"), Asc("L"), Asc("M"), Asc("N"), Asc("O"), Asc("P"), Asc("Q"), Asc("R"), Asc("S"), Asc("T"), Asc("U"), Asc("V"), Asc("W"), Asc("X"), Asc("Y"), Asc("Z"), Asc("-"), Asc("."), Asc(" "), Asc("$"), Asc("/"), Asc("+"), Asc("%"))
   
    Dim RVal As Integer
    RVal = -1
    For x = 0 To (43 - 1)
        If CODE39MAP(x) = inputchar Then
            RVal = x
            Exit For
        End If
    Next x
    
    getCode39Value = RVal

End Function



Public Function filterInput_Code39(ByVal data As String) As String
  
  Dim result As String
  result = ""
  datalength = Len(data)

  Dim barcodechar As String
  For x = 0 To datalength - 1
        barcodechar = Mid(data, x + 1, 1)
        If getCode39Value(AscW(barcodechar)) <> -1 Then
            result = result & barcodechar
        End If
  Next x

  filterInput_Code39 = result

End Function


Public Function getCode39Character(ByVal inputdecimal As Integer) As String

    Dim CODE39MAP() As Variant
    CODE39MAP = Array(Asc("0"), Asc("1"), Asc("2"), Asc("3"), Asc("4"), Asc("5"), Asc("6"), Asc("7"), Asc("8"), Asc("9"), Asc("A"), Asc("B"), Asc("C"), Asc("D"), Asc("E"), Asc("F"), Asc("G"), Asc("H"), Asc("I"), Asc("J"), Asc("K"), Asc("L"), Asc("M"), Asc("N"), Asc("O"), Asc("P"), Asc("Q"), Asc("R"), Asc("S"), Asc("T"), Asc("U"), Asc("V"), Asc("W"), Asc("X"), Asc("Y"), Asc("Z"), Asc("-"), Asc("."), Asc(" "), Asc("$"), Asc("/"), Asc("+"), Asc("%"))

    getCode39Character = CODE39MAP(inputdecimal)

End Function



Public Function generateCheckDigit_Code39(ByVal data As String) As String


  datalength = 0
  sum = 0
  result = -1
  strResult = ""
  Dim barcodechar As String
  
  datalength = Len(data)
  For x = 0 To datalength - 1
        barcodechar = Mid(data, x + 1, 1)
        sum = sum + getCode39Value(AscW(barcodechar))
        
  Next x
  
  result = sum Mod 43
  
  strResult = Chr(getCode39Character(result))
  generateCheckDigit_Code39 = strResult

End Function

'======================================================================================================
'EAN8
'======================================================================================================

Public Function Encode_EAN8(ByVal data As String) As String

    Dim result As String
    Dim cd As String
    Dim filtereddata As String
          
    transformdataleft = ""
    transformdataright = ""
    filtereddata = filterInput_EAN8(data)
    filteredlength = Len(filtereddata)

    If filteredlength > 7 Then
        filtereddata = left$(filtereddata, 7)
    End If

    If filteredlength < 7 Then
    
        addcharlength = 7 - Len(filtereddata)
        For x = 0 To addcharlength - 1
            filtereddata = "0" & filtereddata
        Next x
    
    End If

    cd = generateCheckDigit_EAN8(filtereddata)

    'Transformation
    filtereddata = filtereddata + cd
    datalength = 0
    datalength = Len(filtereddata)

    For x = 0 To 4 - 1
        transformdataleft = transformdataleft + Mid(filtereddata, x + 1, 1)
        
    Next x
    
    transformchar = ""
    For x = 4 To 8 - 1
    
        transformchar = Mid(filtereddata, x + 1, 1)
        
        transformchar = Chr(AscW(transformchar) + 49)
        
        transformdataright = transformdataright + transformchar
    
    Next x

    'hr not used.
    result = "[" + transformdataleft + "-" + transformdataright + "]"
    Encode_EAN8 = result

End Function




Public Function generateCheckDigit_EAN8(ByVal data As String) As String

  datalength = 0
  parity = 0 '0-odd, 1-even
  sum = 0
  result = -1
  strResult = ""
  barcodechar = ""
  barcodevalue = 0

  datalength = Len(data)

  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = (AscW(barcodechar) - 48)

    If parity = 0 Then 'odd
    
        sum = sum + (3 * barcodevalue)
        parity = 1
    
    Else
        sum = sum + barcodevalue
        parity = 0
    End If
  Next x

  result = sum Mod 10
  If result = 0 Then
    result = 0
  Else
    result = 10 - result
  End If
  
  strResult = Chr(result + AscW("0"))
  generateCheckDigit_EAN8 = strResult

End Function


Public Function filterInput_EAN8(ByVal data As String) As String

  result = ""
  datalength = Len(data)
  
  barcodechar = ""
  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
         result = result + barcodechar
    End If
  Next x

  filterInput_EAN8 = result


End Function

'======================================================================================================
'EAN13
'======================================================================================================

Public Function Encode_EAN13(ByVal data As String, Optional ByVal hr As Integer = 0) As String

    Dim EAN13PARITYMAP(10, 6) As Integer

    
    EAN13PARITYMAP(0, 0) = 0
    EAN13PARITYMAP(0, 1) = 0
    EAN13PARITYMAP(0, 2) = 0
    EAN13PARITYMAP(0, 3) = 0
    EAN13PARITYMAP(0, 4) = 0
    EAN13PARITYMAP(0, 5) = 0
    
    EAN13PARITYMAP(1, 0) = 0
    EAN13PARITYMAP(1, 1) = 0
    EAN13PARITYMAP(1, 2) = 1
    EAN13PARITYMAP(1, 3) = 0
    EAN13PARITYMAP(1, 4) = 1
    EAN13PARITYMAP(1, 5) = 1
    
    EAN13PARITYMAP(2, 0) = 0
    EAN13PARITYMAP(2, 1) = 0
    EAN13PARITYMAP(2, 2) = 1
    EAN13PARITYMAP(2, 3) = 1
    EAN13PARITYMAP(2, 4) = 0
    EAN13PARITYMAP(2, 5) = 1
    
    EAN13PARITYMAP(3, 0) = 0
    EAN13PARITYMAP(3, 1) = 0
    EAN13PARITYMAP(3, 2) = 1
    EAN13PARITYMAP(3, 3) = 1
    EAN13PARITYMAP(3, 4) = 1
    EAN13PARITYMAP(3, 5) = 0
    
    EAN13PARITYMAP(4, 0) = 0
    EAN13PARITYMAP(4, 1) = 1
    EAN13PARITYMAP(4, 2) = 0
    EAN13PARITYMAP(4, 3) = 0
    EAN13PARITYMAP(4, 4) = 1
    EAN13PARITYMAP(4, 5) = 1
    
    EAN13PARITYMAP(5, 0) = 0
    EAN13PARITYMAP(5, 1) = 1
    EAN13PARITYMAP(5, 2) = 1
    EAN13PARITYMAP(5, 3) = 0
    EAN13PARITYMAP(5, 4) = 0
    EAN13PARITYMAP(5, 5) = 1
    
    EAN13PARITYMAP(6, 0) = 0
    EAN13PARITYMAP(6, 1) = 1
    EAN13PARITYMAP(6, 2) = 1
    EAN13PARITYMAP(6, 3) = 1
    EAN13PARITYMAP(6, 4) = 0
    EAN13PARITYMAP(6, 5) = 0
    
    EAN13PARITYMAP(7, 0) = 0
    EAN13PARITYMAP(7, 1) = 1
    EAN13PARITYMAP(7, 2) = 0
    EAN13PARITYMAP(7, 3) = 1
    EAN13PARITYMAP(7, 4) = 0
    EAN13PARITYMAP(7, 5) = 1
    
    EAN13PARITYMAP(8, 0) = 0
    EAN13PARITYMAP(8, 1) = 1
    EAN13PARITYMAP(8, 2) = 0
    EAN13PARITYMAP(8, 3) = 1
    EAN13PARITYMAP(8, 4) = 1
    EAN13PARITYMAP(8, 5) = 0
    
    EAN13PARITYMAP(9, 0) = 0
    EAN13PARITYMAP(9, 1) = 1
    EAN13PARITYMAP(9, 2) = 1
    EAN13PARITYMAP(9, 3) = 0
    EAN13PARITYMAP(9, 4) = 1
    EAN13PARITYMAP(9, 5) = 0


    Dim result As String
    Dim cd As String
    Dim filtereddata As String
          
    transformdataleft = ""
    transformdataright = ""
    filtereddata = filterInput_EAN13(data)
    filteredlength = Len(filtereddata)
    

    If filteredlength > 12 Then
        filtereddata = left$(filtereddata, 12)
        
    End If

    If filteredlength < 12 Then
            
        addcharlength = 12 - Len(filtereddata)
        For x = 0 To addcharlength - 1
            filtereddata = "0" & filtereddata
        Next x
    
    End If

    cd = generateCheckDigit_EAN13(filtereddata)
    
    
    'Transformation
    filtereddata = filtereddata + cd
    datalength = 0
    datalength = Len(filtereddata)

    parityBit = 0
    firstdigit = 0
    For x = 0 To 7 - 1
        
        
        If x = 0 Then
            firstdigit = AscW(Mid(filtereddata, x + 1, 1)) - AscW("0") 'First digit is used for parity
            
        Else
        
            parityBit = EAN13PARITYMAP(firstdigit, x - 1)
            If parityBit = 0 Then 'odd
                transformdataleft = transformdataleft + Mid(filtereddata, x + 1, 1)
            Else
                transformdataleft = transformdataleft + Chr(AscW(Mid(filtereddata, x + 1, 1)) + 49 + 14)
            End If
            
        End If
        
    Next x
    
    transformchar = ""
    For x = 7 To 13 - 1 'include check digit
    
        transformchar = Mid(filtereddata, x + 1, 1)
        transformchar = Chr(AscW(transformchar) + 49)
        transformdataright = transformdataright + transformchar
        
    Next x

    If hr = 1 Then
        result = Chr(firstdigit + AscW("!")) + "[" + transformdataleft + "-" + transformdataright + "]"
    Else
        result = "[" + transformdataleft + "-" + transformdataright + "]"
    End If
    
    Encode_EAN13 = result
    

End Function




Public Function generateCheckDigit_EAN13(ByVal data As String) As String
  
  datalength = 0
  parity = 0 '0-odd, 1-even
  sum = 0
  result = -1
  strResult = ""
  barcodechar = ""
  barcodevalue = 0

  datalength = Len(data)

  For x = (datalength - 1) To 0 Step -1
  
     barcodechar = Mid(data, x + 1, 1)
     barcodevalue = (AscW(barcodechar) - 48)

    If (x Mod 2) = 1 Then
        sum = sum + (3 * barcodevalue)
    
    Else
    
        sum = sum + barcodevalue
    End If
  
  Next x


  result = sum Mod 10
  If result = 0 Then
    result = 0
  Else
    result = 10 - result
  End If
  
  strResult = Chr(result + AscW("0"))
  generateCheckDigit_EAN13 = strResult



End Function



Public Function filterInput_EAN13(ByVal data As String) As String
  
  result = ""
  datalength = Len(data)
  
  barcodechar = ""
  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
         result = result + barcodechar
    End If
  Next x

  filterInput_EAN13 = result

End Function

'======================================================================================================
'EXT2
'======================================================================================================

Public Function filterInput_EXT2(ByVal data As String) As String

  result = ""
  datalength = Len(data)
  
   barcodechar = ""
  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
        
         result = result + barcodechar
    End If
  Next x

  
  filterInput_EXT2 = result

End Function


Public Function Encode_EXT2(ByVal data As String) As String

    Dim EXT2PARITYMAP(4, 2) As Variant
    EXT2PARITYMAP(0, 0) = 0
    EXT2PARITYMAP(0, 1) = 0
    
    EXT2PARITYMAP(1, 0) = 0
    EXT2PARITYMAP(1, 1) = 1
    
    EXT2PARITYMAP(2, 0) = 1
    EXT2PARITYMAP(2, 1) = 0
    
    EXT2PARITYMAP(3, 0) = 1
    EXT2PARITYMAP(3, 1) = 1

    Dim result As String
    Dim cd As String
    Dim filtereddata As String
          
    transformdataleft = ""
    transformdataright = ""
    filtereddata = filterInput_EXT2(data)
    filteredlength = Len(filtereddata)
    
    If filteredlength > 2 Then
        filtereddata = left$(filtereddata, 2)
    End If

    If filteredlength < 2 Then
    
        addcharlength = 2 - Len(filtereddata)
        For x = 0 To addcharlength - 1
            filtereddata = "0" & filtereddata
        Next x
    
    End If
    
    
    'Transformation
    sum = 0
    value1 = 0
    value2 = 0
    parityindex = 0
    value1 = (AscW(Mid(filtereddata, 0 + 1, 1)) - 48) * 10
    value2 = (AscW(Mid(filtereddata, 1 + 1, 1)) - 48)
    sum = value1 + value2
    parityindex = sum Mod 4

    datalength = 0
    datalength = Len(filtereddata)

    parityBit = 0
    parityBit = EXT2PARITYMAP(parityindex, 0)
    If parityBit = 0 Then 'odd
        transformdataleft = transformdataleft + Mid(filtereddata, 0 + 1, 1)
    Else
        transformdataleft = transformdataleft + Chr(AscW(Mid(filtereddata, 0 + 1, 1)) + 49 + 14)
    End If

    parityBit = EXT2PARITYMAP(parityindex, 1)
    If parityBit = 0 Then 'odd
        transformdataright = transformdataright + Mid(filtereddata, 1 + 1, 1)
    Else
        transformdataright = transformdataright + Chr(AscW(Mid(filtereddata, 1 + 1, 1)) + 49 + 14)
    End If

    result = "<" + transformdataleft + "+" + transformdataright
    Encode_EXT2 = result
    

End Function

'======================================================================================================
'EXT5
'======================================================================================================

Public Function filterInput_EXT5(ByVal data As String) As String

    result = ""
    datalength = Len(data)
  
    barcodechar = ""
    For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
        
         result = result + barcodechar
    End If
    Next x
  
    filterInput_EXT5 = result

End Function


Public Function Encode_EXT5(ByVal data As String) As String

    Dim EXT5PARITYMAP(10, 5) As Variant
    EXT5PARITYMAP(0, 0) = 1
    EXT5PARITYMAP(0, 1) = 1
    EXT5PARITYMAP(0, 2) = 0
    EXT5PARITYMAP(0, 3) = 0
    EXT5PARITYMAP(0, 4) = 0
    
    EXT5PARITYMAP(1, 0) = 1
    EXT5PARITYMAP(1, 1) = 0
    EXT5PARITYMAP(1, 2) = 1
    EXT5PARITYMAP(1, 3) = 0
    EXT5PARITYMAP(1, 4) = 0
    
    EXT5PARITYMAP(2, 0) = 1
    EXT5PARITYMAP(2, 1) = 0
    EXT5PARITYMAP(2, 2) = 0
    EXT5PARITYMAP(2, 3) = 1
    EXT5PARITYMAP(2, 4) = 0
    
    EXT5PARITYMAP(3, 0) = 1
    EXT5PARITYMAP(3, 1) = 0
    EXT5PARITYMAP(3, 2) = 0
    EXT5PARITYMAP(3, 3) = 0
    EXT5PARITYMAP(3, 4) = 1
    
    EXT5PARITYMAP(4, 0) = 0
    EXT5PARITYMAP(4, 1) = 1
    EXT5PARITYMAP(4, 2) = 1
    EXT5PARITYMAP(4, 3) = 0
    EXT5PARITYMAP(4, 4) = 0
    
    EXT5PARITYMAP(5, 0) = 0
    EXT5PARITYMAP(5, 1) = 0
    EXT5PARITYMAP(5, 2) = 1
    EXT5PARITYMAP(5, 3) = 1
    EXT5PARITYMAP(5, 4) = 0
    
    EXT5PARITYMAP(6, 0) = 0
    EXT5PARITYMAP(6, 1) = 0
    EXT5PARITYMAP(6, 2) = 0
    EXT5PARITYMAP(6, 3) = 1
    EXT5PARITYMAP(6, 4) = 1
    
    EXT5PARITYMAP(7, 0) = 0
    EXT5PARITYMAP(7, 1) = 1
    EXT5PARITYMAP(7, 2) = 0
    EXT5PARITYMAP(7, 3) = 1
    EXT5PARITYMAP(7, 4) = 0
    
    EXT5PARITYMAP(8, 0) = 0
    EXT5PARITYMAP(8, 1) = 1
    EXT5PARITYMAP(8, 2) = 0
    EXT5PARITYMAP(8, 3) = 0
    EXT5PARITYMAP(8, 4) = 1
    
    EXT5PARITYMAP(9, 0) = 0
    EXT5PARITYMAP(9, 1) = 0
    EXT5PARITYMAP(9, 2) = 1
    EXT5PARITYMAP(9, 3) = 0
    EXT5PARITYMAP(9, 4) = 1

    Dim result As String
    Dim cd As String
    Dim filtereddata As String
          
    transformdata = ""
    filtereddata = filterInput_EXT5(data)
    filteredlength = Len(filtereddata)
    
    If filteredlength > 5 Then
        filtereddata = left$(filtereddata, 5)
    End If

    If filteredlength < 5 Then
        
        addcharlength = 5 - Len(filtereddata)
        For x = 0 To addcharlength - 1
            filtereddata = "0" & filtereddata
        Next x
    End If

    'Transformation
    sum = 0
    datalength = 0
    datalength = Len(filtereddata)

    barcodechar = ""
    barcodevalue = 0
    For x = datalength - 1 To 0 Step -1
    
      barcodechar = Mid(filtereddata, x + 1, 1)
      barcodevalue = AscW(barcodechar)
      
      If x Mod 2 = 0 Then
        sum = sum + (3 * (barcodevalue - 48))
      Else
        sum = sum + (9 * (barcodevalue - 48))
      End If
    Next x
    
    chk = sum Mod 10

    
    For x = 0 To 5 - 1
     
        parityBit = 0
        parityBit = EXT5PARITYMAP(chk, x)
        If parityBit = 0 Then 'odd
            transformdata = transformdata + Mid(filtereddata, x + 1, 1)
        Else
            transformdata = transformdata + Chr(AscW(Mid(filtereddata, x + 1, 1)) + 49 + 14)
        End If
    
    Next x

    result = "<" + Mid(transformdata, 0 + 1, 1) + "+" + Mid(transformdata, 1 + 1, 1) + "+" + Mid(transformdata, 2 + 1, 1) + "+" + Mid(transformdata, 3 + 1, 1) + "+" + Mid(transformdata, 4 + 1, 1)
    Encode_EXT5 = result

End Function

'======================================================================================================
'UPCA
'======================================================================================================

Public Function Encode_UPCA(ByVal data As String, Optional ByVal hr As Integer = 0) As String

    cd = ""
    result = ""
    filtereddata = filterInput_UPCA(data)
    transformdataleft = ""
    transformdataright = ""

    filteredlength = Len(filtereddata)

    If (filteredlength > 11) Then
        filtereddata = left$(filtereddata, 11)
    End If

    If (filteredlength < 11) Then
    
        addcharlength = 11 - Len(filtereddata)
        For x = 0 To addcharlength - 1
            filtereddata = "0" & filtereddata
        Next x
    End If

    cd = generateCheckDigit_UPCA(filtereddata)

    'Transformation
    filtereddata = filtereddata + cd
    datalength = 0
    datalength = Len(filtereddata)
    For x = 0 To 6 - 1
        transformdataleft = transformdataleft + Mid(filtereddata, x + 1, 1)
    Next x

    For x = 6 To 12 - 1
        
        transformchar = AscW(Mid(filtereddata, x + 1, 1))
        transformchar = transformchar + 49 'Right Parity Characters transform 0 to a etc...
        transformdataright = transformdataright + Chr(transformchar)
    Next x
    
    If (hr = 1) Then 'make first and last digit use long bars. add 110
        result = Chr(AscW(Mid(transformdataleft, 0 + 1, 1)) - 15) + "[" + Chr(AscW(Mid(transformdataleft, 0 + 1, 1)) + 110) + Mid(transformdataleft, 1 + 1, 5) + "-" + Mid(transformdataright, 0 + 1, 5) + Chr(AscW(Mid(transformdataright, 5 + 1, 1)) - 49 + 159) + "]" + Chr(AscW(Mid(transformdataright, 5 + 1, 1)) - 49 - 15)
    Else
        result = "[" + transformdataleft + "-" + transformdataright + "]"
    End If
        
    Encode_UPCA = result


End Function


Public Function generateCheckDigit_UPCA(ByVal data As String) As String

  datalength = 0
  sum = 0
  result = -1
  strResult = ""
  barcodechar = ""
  barcodevalue = 0

  datalength = Len(data)

  For x = 0 To datalength - 1
    
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = (AscW(barcodechar) - 48)

    If (x Mod 2 = 0) Then
        sum = sum + (3 * barcodevalue)
    Else
        sum = sum + barcodevalue
    End If
    
  Next x

  result = sum Mod 10
  If (result = 0) Then
    result = 0
  Else
    result = 10 - result
  End If
  
  strResult = Chr(result + AscW("0"))
  generateCheckDigit_UPCA = strResult

End Function


Public Function getUPCACharacter(ByVal inputdecimal As Integer) As Integer

    getUPCACharacter = (inputdecimal + 48)

End Function

    

Public Function filterInput_UPCA(ByVal data As String) As String

    result = ""
    datalength = Len(data)
  
    barcodechar = ""
    For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
        
         result = result + barcodechar
    End If
    Next x
  
    filterInput_UPCA = result

End Function

'======================================================================================================
'UPCE
'======================================================================================================

Public Function filterInput_UPCE(ByVal data As String) As String

    result = ""
    datalength = Len(data)
  
    barcodechar = ""
    For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
        
         result = result + barcodechar
    End If
    Next x
  
    filterInput_UPCE = result

End Function


Public Function getUPCECharacter(ByVal inputdecimal As Integer) As Integer

    getUPCECharacter = (inputdecimal + 48)

End Function



Public Function generateCheckDigit_UPCE(ByVal data As String) As String

  datalength = 0
  sum = 0
  result = -1
  strResult = ""
  barcodechar = ""
  barcodevalue = 0

  datalength = Len(data)

  For x = 0 To datalength - 1
    
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = (AscW(barcodechar) - 48)
    
    If (parity = 0) Then
    
        sum = sum + (3 * barcodevalue)
        parity = 1
    
    Else
    
        sum = sum + barcodevalue
        parity = 0
    End If
    
  Next x

  result = sum Mod 10
  If (result = 0) Then
    result = 0
  Else
    result = 10 - result
  End If
  
  strResult = Chr(getUPCECharacter(result))
  generateCheckDigit_UPCE = strResult

End Function

 Public Function Encode_UPCE(ByVal data As String, Optional ByVal hr As Integer = 0) As String
 
 
    Dim UPCEPARITYMAP(10, 2, 6) As Integer
    
    UPCEPARITYMAP(0, 0, 0) = 1
    UPCEPARITYMAP(0, 0, 1) = 1
    UPCEPARITYMAP(0, 0, 2) = 1
    UPCEPARITYMAP(0, 0, 3) = 0
    UPCEPARITYMAP(0, 0, 4) = 0
    UPCEPARITYMAP(0, 0, 5) = 0
    
    UPCEPARITYMAP(0, 1, 0) = 0
    UPCEPARITYMAP(0, 1, 1) = 0
    UPCEPARITYMAP(0, 1, 2) = 0
    UPCEPARITYMAP(0, 1, 3) = 1
    UPCEPARITYMAP(0, 1, 4) = 1
    UPCEPARITYMAP(0, 1, 5) = 1
    
    
    UPCEPARITYMAP(1, 0, 0) = 1
    UPCEPARITYMAP(1, 0, 1) = 1
    UPCEPARITYMAP(1, 0, 2) = 0
    UPCEPARITYMAP(1, 0, 3) = 1
    UPCEPARITYMAP(1, 0, 4) = 0
    UPCEPARITYMAP(1, 0, 5) = 0
    
    UPCEPARITYMAP(1, 1, 0) = 0
    UPCEPARITYMAP(1, 1, 1) = 0
    UPCEPARITYMAP(1, 1, 2) = 1
    UPCEPARITYMAP(1, 1, 3) = 0
    UPCEPARITYMAP(1, 1, 4) = 1
    UPCEPARITYMAP(1, 1, 5) = 1
    
    
    UPCEPARITYMAP(2, 0, 0) = 1
    UPCEPARITYMAP(2, 0, 1) = 1
    UPCEPARITYMAP(2, 0, 2) = 0
    UPCEPARITYMAP(2, 0, 3) = 0
    UPCEPARITYMAP(2, 0, 4) = 1
    UPCEPARITYMAP(2, 0, 5) = 0
    
    UPCEPARITYMAP(2, 1, 0) = 0
    UPCEPARITYMAP(2, 1, 1) = 0
    UPCEPARITYMAP(2, 1, 2) = 1
    UPCEPARITYMAP(2, 1, 3) = 1
    UPCEPARITYMAP(2, 1, 4) = 0
    UPCEPARITYMAP(2, 1, 5) = 1
    
    UPCEPARITYMAP(3, 0, 0) = 1
    UPCEPARITYMAP(3, 0, 1) = 1
    UPCEPARITYMAP(3, 0, 2) = 0
    UPCEPARITYMAP(3, 0, 3) = 0
    UPCEPARITYMAP(3, 0, 4) = 0
    UPCEPARITYMAP(3, 0, 5) = 1
    
    UPCEPARITYMAP(3, 1, 0) = 0
    UPCEPARITYMAP(3, 1, 1) = 0
    UPCEPARITYMAP(3, 1, 2) = 1
    UPCEPARITYMAP(3, 1, 3) = 1
    UPCEPARITYMAP(3, 1, 4) = 1
    UPCEPARITYMAP(3, 1, 5) = 0
 
    UPCEPARITYMAP(4, 0, 0) = 1
    UPCEPARITYMAP(4, 0, 1) = 0
    UPCEPARITYMAP(4, 0, 2) = 1
    UPCEPARITYMAP(4, 0, 3) = 1
    UPCEPARITYMAP(4, 0, 4) = 0
    UPCEPARITYMAP(4, 0, 5) = 0
    
    UPCEPARITYMAP(4, 1, 0) = 0
    UPCEPARITYMAP(4, 1, 1) = 1
    UPCEPARITYMAP(4, 1, 2) = 0
    UPCEPARITYMAP(4, 1, 3) = 0
    UPCEPARITYMAP(4, 1, 4) = 1
    UPCEPARITYMAP(4, 1, 5) = 1
    
    UPCEPARITYMAP(5, 0, 0) = 1
    UPCEPARITYMAP(5, 0, 1) = 0
    UPCEPARITYMAP(5, 0, 2) = 0
    UPCEPARITYMAP(5, 0, 3) = 1
    UPCEPARITYMAP(5, 0, 4) = 1
    UPCEPARITYMAP(5, 0, 5) = 0
    
    UPCEPARITYMAP(5, 1, 0) = 0
    UPCEPARITYMAP(5, 1, 1) = 1
    UPCEPARITYMAP(5, 1, 2) = 1
    UPCEPARITYMAP(5, 1, 3) = 0
    UPCEPARITYMAP(5, 1, 4) = 0
    UPCEPARITYMAP(5, 1, 5) = 1
    
    
    UPCEPARITYMAP(6, 0, 0) = 1
    UPCEPARITYMAP(6, 0, 1) = 0
    UPCEPARITYMAP(6, 0, 2) = 0
    UPCEPARITYMAP(6, 0, 3) = 0
    UPCEPARITYMAP(6, 0, 4) = 1
    UPCEPARITYMAP(6, 0, 5) = 1
    
    UPCEPARITYMAP(6, 1, 0) = 0
    UPCEPARITYMAP(6, 1, 1) = 1
    UPCEPARITYMAP(6, 1, 2) = 1
    UPCEPARITYMAP(6, 1, 3) = 1
    UPCEPARITYMAP(6, 1, 4) = 0
    UPCEPARITYMAP(6, 1, 5) = 0
                            
    UPCEPARITYMAP(7, 0, 0) = 1
    UPCEPARITYMAP(7, 0, 1) = 0
    UPCEPARITYMAP(7, 0, 2) = 1
    UPCEPARITYMAP(7, 0, 3) = 0
    UPCEPARITYMAP(7, 0, 4) = 1
    UPCEPARITYMAP(7, 0, 5) = 0
    
    UPCEPARITYMAP(7, 1, 0) = 0
    UPCEPARITYMAP(7, 1, 1) = 1
    UPCEPARITYMAP(7, 1, 2) = 0
    UPCEPARITYMAP(7, 1, 3) = 1
    UPCEPARITYMAP(7, 1, 4) = 0
    UPCEPARITYMAP(7, 1, 5) = 1
    
    UPCEPARITYMAP(8, 0, 0) = 1
    UPCEPARITYMAP(8, 0, 1) = 0
    UPCEPARITYMAP(8, 0, 2) = 1
    UPCEPARITYMAP(8, 0, 3) = 0
    UPCEPARITYMAP(8, 0, 4) = 0
    UPCEPARITYMAP(8, 0, 5) = 1
    
    UPCEPARITYMAP(8, 1, 0) = 0
    UPCEPARITYMAP(8, 1, 1) = 1
    UPCEPARITYMAP(8, 1, 2) = 0
    UPCEPARITYMAP(8, 1, 3) = 1
    UPCEPARITYMAP(8, 1, 4) = 1
    UPCEPARITYMAP(8, 1, 5) = 0
                            
    UPCEPARITYMAP(9, 0, 0) = 1
    UPCEPARITYMAP(9, 0, 1) = 0
    UPCEPARITYMAP(9, 0, 2) = 0
    UPCEPARITYMAP(9, 0, 3) = 1
    UPCEPARITYMAP(9, 0, 4) = 0
    UPCEPARITYMAP(9, 0, 5) = 1
    
    UPCEPARITYMAP(9, 1, 0) = 0
    UPCEPARITYMAP(9, 1, 1) = 1
    UPCEPARITYMAP(9, 1, 2) = 1
    UPCEPARITYMAP(9, 1, 3) = 0
    UPCEPARITYMAP(9, 1, 4) = 1
    UPCEPARITYMAP(9, 1, 5) = 0

 
    upcestr = ""
    cd = ""
    result = ""
    filtereddata = filterInput_UPCE(data)
    transformdata = ""

    filteredlength = Len(filtereddata)

    If (filteredlength > 6) Then
        filtereddata = left$(filtereddata, 6)
    End If

    If (filteredlength < 6) Then
    
        addcharlength = 6 - Len(filtereddata)
        For x = 0 To addcharlength - 1
            filtereddata = "0" & filtereddata
        Next x
        
    End If

    'Transformation
    'Expand
    datalength = 0
    datalength = Len(filtereddata)

    upcastr = "0"
    lastchar = Mid$(filtereddata, datalength - 1 + 1, 1)

    If (AscW(lastchar) = AscW("0")) Or (AscW(lastchar) = AscW("1")) Or (AscW(lastchar) = AscW("2")) Then
    
        
        upcastr = upcastr + Mid$(filtereddata, 0 + 1, 2)
        upcastr = upcastr + lastchar
        upcastr = upcastr + "0000"
        upcastr = upcastr + Mid$(filtereddata, 2 + 1, 3)
    
    ElseIf (AscW(lastchar) = AscW("3")) Then
    
        upcastr = upcastr + Mid$(filtereddata, 0 + 1, 3)
        upcastr = upcastr + "00000"
        upcastr = upcastr + Mid$(filtereddata, 3 + 1, 2)
    
    ElseIf (AscW(lastchar) = AscW("4")) Then
    
        upcastr = upcastr + Mid$(filtereddata, 0 + 1, 4)
        upcastr = upcastr + "00000"
        upcastr = upcastr + Mid$(filtereddata, 4 + 1, 1)
    
    ElseIf ((AscW(lastchar) = AscW("5")) Or (AscW(lastchar) = AscW("6")) Or (AscW(lastchar) = AscW("7")) Or (AscW(lastchar) = AscW("8")) Or (AscW(lastchar) = AscW("9"))) Then
    
        upcastr = upcastr + Mid$(filtereddata, 0 + 1, 5)
        upcastr = upcastr + "0000"
        upcastr = upcastr + lastchar
        
    End If
    
    
    filtereddata = upcastr

    cd = generateCheckDigit_UPCE(filtereddata)

    'Contract
    upcestr = upcastr
    productvalue = val(Mid$(upcastr, 6 + 1, 5))
    If ((Mid$(upcestr, 3 + 1, 3) = "000") Or (Mid$(upcestr, 3 + 1, 3) = "100") Or (Mid$(upcestr, 3 + 1, 3) = "200")) Then
    
        If (productvalue >= 0) And (productvalue <= 999) Then
        
            thirdch = Mid$(upcestr, 3 + 1, 1)
            
            upcestr = DeleteSubString(upcestr, 3 + 1, 5)
            upcestr = InsertSubString(upcestr, thirdch, 6 + 1)
        
        Else
            upcestr = "000000"
        End If
    
    ElseIf (Mid$(upcestr, 4 + 1, 2) = "00") Then
      
        If (productvalue >= 0) And (productvalue <= 99) Then
        
            upcestr = DeleteSubString(upcestr, 4 + 1, 5)
            upcestr = InsertSubString(upcestr, "3", 6 + 1)
        
        Else
            upcestr = "000000"
        End If
    
     ElseIf (Mid$(upcestr, 5 + 1, 1) = "0") Then
      
        If (productvalue >= 0) And (productvalue <= 9) Then
        
            upcestr = DeleteSubString(upcestr, 5 + 1, 5)
            upcestr = InsertSubString(upcestr, "4", 6 + 1)
        
        Else
            upcestr = "000000"
        End If
        
      
     ElseIf (Mid$(upcestr, 5 + 1, 1) <> "0") Then
      
        If ((productvalue >= 5) And (productvalue <= 9)) Then
            upcestr = DeleteSubString(upcestr, 6 + 1, 4)
        
        Else
            upcestr = "000000"
            
        End If
      
     Else
        upcestr = "000000"
        
     End If

    filtereddata = upcestr
  
    parityBit = 0
    nschar = "0"
    chkvalue = (AscW(Mid$(cd, 0 + 1, 1)) - AscW("0"))

    For x = 1 To 7 - 1
    
        nsvalue = (AscW(nschar) - AscW("0"))
        transformchar = Mid$(filtereddata, x + 1, 1)
        parityBit = UPCEPARITYMAP(chkvalue, nsvalue, x - 1)
        If (parityBit = 1) Then
            transformchar = Chr(AscW(transformchar) + 48 + 15)
        End If
        transformdata = transformdata + transformchar
        
    Next x

    If (hr = 1) Then
        result = Chr(AscW(nschar) - 15) + "{" + transformdata + "}" + Chr(chkvalue + AscW("0") - 15)
    Else
        result = "{" + transformdata + "}"
    End If
    
    Encode_UPCE = result

 End Function



Public Function DeleteSubString(ByVal orgStr As String, ByVal start As Integer, ByVal length As Integer) As String


    fulllength = Len(orgStr)
    leftstr = ""
    rightstr = ""
    If (start < 1) Then
        start = 1
    End If
    
    leftstr = left$(orgStr, start - 1)
    rightstr = right$(orgStr, fulllength - (start + length) + 1)
    DeleteSubString = leftstr + rightstr

End Function


Public Function InsertSubString(ByVal orgStr As String, ByVal insertStr As String, ByVal start As Integer) As String

    fulllength = Len(orgStr)
    leftstr = ""
    rightstr = ""
    
    If (start < 1) Then
        start = 1
    End If
    
    If (start > fulllength + 1) Then
        start = fulllength + 1
    End If
    
    leftstr = left$(orgStr, start - 1)
    rightstr = right$(orgStr, fulllength - start + 1)
    InsertSubString = leftstr + insertStr + rightstr

End Function

'======================================================================================================
'Code128A
'======================================================================================================

Public Function filterInput_Code128A(ByVal data As String) As String

    result = ""
    datalength = Len(data)
  
    barcodechar = ""
    For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = AscW(barcodechar)
    
    If (barcodevalue <= 95) And (barcodevalue >= 0) Then
        result = result + barcodechar
    End If
    
    Next x
  
    filterInput_Code128A = result

End Function


Public Function translateCharacter(ByVal inputchar As Integer) As String

    returnvalue = 0

    If (inputchar <= 30) And (inputchar >= 0) Then
        returnvalue = (inputchar + 96)
    ElseIf (inputchar = 31) Then
        returnvalue = (inputchar + 96 + 100)
    ElseIf (inputchar <= 95) And (inputchar >= 32) Then
        returnvalue = inputchar
    Else
        returnvalue = -1
    End If

    translateCharacter = Chr(returnvalue)

End Function



Public Function getCode128AValue(ByVal inputchar As Integer) As Integer

    returnvalue = 0

    If (inputchar <= 31) And (inputchar >= 0) Then
        returnvalue = (inputchar + 64)
    ElseIf (inputchar <= 95) And (inputchar >= 32) Then
        returnvalue = (inputchar - 32)
    Else
        returnvalue = -1
    End If

    getCode128AValue = returnvalue

End Function


Public Function getCode128ACharacter(ByVal inputvalue As Integer) As String

    If (inputvalue <= 94) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 106) And (inputvalue >= 95) Then
        inputvalue = inputvalue + 100 + 32
    Else
        inputvalue = -1
    End If

    getCode128ACharacter = Chr(inputvalue)


End Function


Public Function generateCheckDigit_Code128A(ByVal data As String) As String

  datalength = 0
  sum = 103
  result = -1
  strResult = ""

  datalength = Len(data)

  x = 0
  weight = 1
  
  For x = 0 To Len(data) - 1
  
    c = AscW(Mid$(data, x + 1, 1))
    sum = sum + (getCode128AValue(c) * (weight))
    weight = weight + 1
    
  
  Next x

  result = sum Mod 103
  strResult = getCode128ACharacter(result)
  

  generateCheckDigit_Code128A = strResult

End Function


 Public Function Encode_Code128A(ByVal data As String) As String
 
    cd = ""
    result = ""
    filtereddata = filterInput_Code128A(data)
    filteredlength = Len(filtereddata)
    If (filteredlength > 254) Then
        filtereddata = left$(filtereddata, 254)
    End If

    cd = generateCheckDigit_Code128A(filtereddata)

    x = 0
  
    For x = 0 To Len(filtereddata) - 1
  
      c = translateCharacter(AscW(Mid$(filtereddata, x + 1, 1)))
      result = result + c
  
    Next x

    result = result + cd
    startc = 235
    stopc = 238
    result = Chr(startc) + result + Chr(stopc)
    Encode_Code128A = result

 End Function

'======================================================================================================
'Code128B
'======================================================================================================

Public Function filterInput_Code128B(ByVal data As String) As String

    result = ""
    datalength = Len(data)
  
    barcodechar = ""
    For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = AscW(barcodechar)
    
    If (barcodevalue <= 127) And (barcodevalue >= 32) Then
        result = result + barcodechar
    End If
    
    Next x
  
    filterInput_Code128B = result

End Function


Public Function getCode128BValue(ByVal inputchar As Integer) As Integer

    returnvalue = 0

    If (inputchar <= 127) And (inputchar >= 32) Then
        returnvalue = (inputchar - 32)
    Else
        returnvalue = -1
    End If

    getCode128BValue = returnvalue

End Function



Public Function getCode128BCharacter(ByVal inputvalue As Integer) As String

    If (inputvalue <= 94) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 106) And (inputvalue >= 95) Then
        inputvalue = inputvalue + 100 + 32
    Else
        inputvalue = -1
    End If

    getCode128BCharacter = Chr(inputvalue)


End Function







Public Function generateCheckDigit_Code128B(ByVal data As String) As String

  datalength = 0
  sum = 104
  result = -1
  strResult = ""

  datalength = Len(data)

  x = 0
  weight = 1
  num = 0
  
  For x = 0 To Len(data) - 1
    
    num = AscW(Mid$(data, x + 1, 1))
    sum = sum + (getCode128BValue(num) * (weight))
    weight = weight + 1
  
  Next x

  result = sum Mod 103
  strResult = getCode128BCharacter(result)
  generateCheckDigit_Code128B = strResult

End Function


 Public Function Encode_Code128B(ByVal data As String) As String
 
    cd = ""
    result = ""
    filtereddata = filterInput_Code128B(data)
    filteredlength = Len(filtereddata)
    If (filteredlength > 254) Then
        filtereddata = left$(filtereddata, 254)
    End If

    cd = generateCheckDigit_Code128B(filtereddata)

    x = 0
    For x = 0 To Len(filtereddata) - 1
  
      c = AscW(Mid$(filtereddata, x + 1, 1))
        If (c = 127) Then
        c = c + 100
        Else
        c = c
        End If

      result = result + Chr(c)
  
    Next x

    result = result + cd

    startc = 236
    stopc = 238
    result = Chr(startc) + result + Chr(stopc)
    Encode_Code128B = result

 End Function

'======================================================================================================
'Code128C
'======================================================================================================

Public Function filterInput_Code128C(ByVal data As String) As String

  result = ""
  datalength = Len(data)
  
  barcodechar = ""
  For x = 0 To datalength - 1
    
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
         result = result + barcodechar
    End If
  Next x

  filterInput_Code128C = result


End Function
  
  

Public Function getCode128CCharacter(ByVal inputvalue As Integer) As String

    If (inputvalue <= 94) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 106) And (inputvalue >= 95) Then
        inputvalue = inputvalue + 32 + 100
    Else
        inputvalue = -1
    End If

    getCode128CCharacter = Chr(inputvalue)


End Function



Public Function generateCheckDigit_Code128C(ByVal data As String) As String

  datalength = 0
  sum = 105
  num = 0
  result = -1
  strResult = ""

  datalength = Len(data)

  x = 0
  weight = 1
  
  For x = 0 To Len(data) - 1 Step 2
    
    num = val(Mid$(data, x + 1, 2))
    sum = sum + (num * weight)
    weight = weight + 1
  
  
  Next x

  result = sum Mod 103
  strResult = getCode128CCharacter(result)
  generateCheckDigit_Code128C = strResult
  

End Function




 Public Function Encode_Code128C(ByVal data As String) As String
 
    num = 0
    cd = ""
    result = ""
    filtereddata = filterInput_Code128C(data)
    filteredlength = Len(filtereddata)
    
    If (filteredlength > 253) Then
        filtereddata = left$(filtereddata, 253)
    End If

    If (Len(filtereddata) Mod 2 = 1) Then
        filtereddata = "0" + filtereddata
    End If
    
    cd = generateCheckDigit_Code128C(filtereddata)
    
    x = 0
    For x = 0 To Len(filtereddata) - 1 Step 2
    
        num = val(Mid$(filtereddata, x + 1, 2))
        result = result + getCode128CCharacter(num)
    
    
    Next x

    result = result + cd

    startc = 237
    stopc = 238
    result = Chr(startc) + result + Chr(stopc)
    Encode_Code128C = result


 End Function
    
'======================================================================================================
'I2of5
'======================================================================================================
    
Public Function filterInput_I2of5(ByVal data As String) As String

    result = ""
    datalength = Len(data)
   
    barcodechar = ""
    For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
         result = result + barcodechar
    End If
    Next x
  
    filterInput_I2of5 = result

End Function

Public Function getI2of5Character(ByVal inputvalue As Integer) As String

    If (inputvalue <= 90) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 99) And (inputvalue >= 91) Then
        inputvalue = inputvalue + 100
    Else
        inputvalue = -1
    End If

    getI2of5Character = Chr(inputvalue)

End Function

Public Function generateCheckDigit_I2of5(ByVal data As String) As String

  datalength = 0
  lastcharpos = 0
  result = 0
  strResult = ""
  barcodechar = ""
  barcodevalue = 0
  toggle = 1
  sum = 0


  datalength = Len(data)
  lastcharpos = datalength - 1

  For x = lastcharpos To 0 Step -1
  
      barcodechar = Mid(data, x + 1, 1)
      barcodevalue = (AscW(barcodechar) - 48)
  
        
      If toggle = 1 Then
            sum = sum + (barcodevalue * 3)
            toggle = 0
      Else
            sum = sum + barcodevalue
            toggle = 1
      End If
  Next x

  If ((sum Mod 10) = 0) Then
        result = AscW("0")
  Else
        result = (10 - (sum Mod 10)) + AscW("0")
  End If

  strResult = Chr(result)
  generateCheckDigit_I2of5 = strResult


End Function


Public Function Encode_I2of5(ByVal data As String, Optional ByVal chk As Integer = 0) As String

    Dim result As String
    Dim cd As String
    Dim filtereddata As String
          
    transformdata = ""
    filtereddata = filterInput_I2of5(data)
    filteredlength = Len(filtereddata)
    
    'Input String including check digit must be even digit numbers in I2of5
    If chk = 1 Then

        If filteredlength > 253 Then
            filtereddata = left$(filtereddata, 253)
            
        End If

        If (Len(filtereddata) Mod 2 = 0) Then   'Make it odd number of digits
            filtereddata = "0" + filtereddata
        End If

        cd = generateCheckDigit_I2of5(filtereddata)  'Together with check digit, it is even number of digits
    
    Else
        
        If filteredlength > 254 Then
            filtereddata = left$(filtereddata, 254)
        End If

        If (Len(filtereddata) Mod 2 = 1) Then 'Make it even number of digits
            filtereddata = "0" + filtereddata
         End If
    
    End If

    filtereddata = filtereddata + cd

    
    num = 0
    For x = 0 To Len(filtereddata) - 1 Step 2
    
        num = val(Mid(filtereddata, x + 1, 2))
        result = result + getI2of5Character(num)
    
    Next x
    result = "{" + result + "}"
    Encode_I2of5 = result

End Function

'======================================================================================================
'Code39ASCII
'======================================================================================================

Public Function Encode_Code39ASCII(ByVal data As String, Optional ByVal chk As Integer = 1) As String

        
    Dim result As String
    Dim cd As String
    Dim filtereddata As String
    result = ""
    cd = ""
            
    filtereddata = filterInput_Code39ASCII(data)
    filteredlength = Len(filtereddata)

    If chk = 1 Then
    
        If filteredlength > 254 Then
        
            filtereddata = left$(filtereddata, 254)
            
        End If
        filtereddata = getCode39ASCIIMappedString(filtereddata)
        cd = generateCheckDigit_Code39ASCII(filtereddata)
    
    Else
    
        If filteredlength > 255 Then
            
            filtereddata = left$(filtereddata, 255)
            
        End If
        filtereddata = getCode39ASCIIMappedString(filtereddata)
    
    End If

    result = "*" + filtereddata + cd + "*"
    Encode_Code39ASCII = result

End Function



Public Function getCode39ASCIIMappedString(ByVal inputx As String) As String
    
    Dim CODE39ASCIIMAP() As Variant
    CODE39ASCIIMAP = Array("%U", "$A", "$B", "$C", "$D", "$E", "$F", "$G", "$H", "$I", "$J", "$K", "$L", "$M", "$N", "$O", "$P", "$Q", "$R", "$S", "$T", "$U", "$V", "$W", "$X", "$Y", "$Z", "%A", "%B", "%C", "%D", "%E", " ", "/A", "/B", "/C", "/D", "/E", "/F", "/G", "/H", "/I", "/J", "/K", "/L", "-", ".", "/O", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "/Z", "%F", "%G", "%H", "%I", "%J", "%V", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "%K", "%L", "%M", "%N", "%O", "%W", "+A", "+B", "+C", "+D", "+E", "+F", "+G", "+H", "+I", "+J", "+K", "+L", "+M", "+N", "+O", "+P", "+Q", "+R", "+S", "+T", "+U", "+V", "+W", "+X", "+Y", "+Z", "%P", "%Q", "%R", "%S", "%T")

    strResult = ""
    datalength = 0
    datalength = Len(inputx)
    For x = 0 To datalength - 1
    
        strResult = strResult + CODE39ASCIIMAP(Asc(Mid$(inputx, x + 1, 1)))
        
    Next x
 
    getCode39ASCIIMappedString = strResult
    

End Function
    
Public Function generateCheckDigit_Code39ASCII(ByVal data As String) As String

  datalength = 0
  sum = 0
  result = -1
  strResult = ""
  Dim barcodechar As String
  
  datalength = Len(data)
  For x = 0 To datalength - 1
        barcodechar = Mid(data, x + 1, 1)
        sum = sum + getCode39ASCIIValue(Asc(barcodechar))
  Next x
  
  result = sum Mod 43
  
  strResult = Chr(getCode39ASCIICharacter(result))
  generateCheckDigit_Code39ASCII = strResult

End Function
    
    
    

Public Function getCode39ASCIICharacter(ByVal inputdecimal As Integer) As String

    Dim CODE39ORIMAP() As Variant
    CODE39ORIMAP = Array(Asc("0"), Asc("1"), Asc("2"), Asc("3"), Asc("4"), Asc("5"), Asc("6"), Asc("7"), Asc("8"), Asc("9"), Asc("A"), Asc("B"), Asc("C"), Asc("D"), Asc("E"), Asc("F"), Asc("G"), Asc("H"), Asc("I"), Asc("J"), Asc("K"), Asc("L"), Asc("M"), Asc("N"), Asc("O"), Asc("P"), Asc("Q"), Asc("R"), Asc("S"), Asc("T"), Asc("U"), Asc("V"), Asc("W"), Asc("X"), Asc("Y"), Asc("Z"), Asc("-"), Asc("."), Asc(" "), Asc("$"), Asc("/"), Asc("+"), Asc("%"))

    getCode39ASCIICharacter = CODE39ORIMAP(inputdecimal)

End Function



Public Function getCode39ASCIIValue(ByVal inputchar As Integer) As Integer

        
    Dim CODE39ORIMAP() As Variant
    CODE39ORIMAP = Array(Asc("0"), Asc("1"), Asc("2"), Asc("3"), Asc("4"), Asc("5"), Asc("6"), Asc("7"), Asc("8"), Asc("9"), Asc("A"), Asc("B"), Asc("C"), Asc("D"), Asc("E"), Asc("F"), Asc("G"), Asc("H"), Asc("I"), Asc("J"), Asc("K"), Asc("L"), Asc("M"), Asc("N"), Asc("O"), Asc("P"), Asc("Q"), Asc("R"), Asc("S"), Asc("T"), Asc("U"), Asc("V"), Asc("W"), Asc("X"), Asc("Y"), Asc("Z"), Asc("-"), Asc("."), Asc(" "), Asc("$"), Asc("/"), Asc("+"), Asc("%"))
 
    Dim RVal As Integer
    RVal = -1
    For x = 0 To (43 - 1)
        If CODE39ORIMAP(x) = inputchar Then
            RVal = x
            Exit For
        End If
    Next x
    
    getCode39ASCIIValue = RVal

End Function



Public Function filterInput_Code39ASCII(ByVal data As String) As String

  
  Dim result As String
  result = ""
  datalength = Len(data)

  Dim barcodechar As String
  For x = 0 To datalength - 1
        barcodechar = Mid(data, x + 1, 1)
        barcodeasciivalue = Asc(barcodechar)
        If (barcodeasciivalue >= 0) And (barcodeasciivalue <= 127) Then
            result = result & barcodechar
        End If
  Next x

  filterInput_Code39ASCII = result

End Function

'==========================================================================
'Encode_UCCEAN
'==========================================================================
Public Function Encode_UCCEAN(ByVal data As String, Optional ByVal GS1Compliance As Integer = 0) As String

    cd = ""
    result = ""
    replacedata102 = ""
    replacedata234 = ""
    filtereddata = filterInputUCCEAN(data)
    filteredlength = Len(filtereddata)
    If filteredlength > 254 Then
            filtereddata = left$(filtereddata, 254)
    End If
    
    replacedata234 = replaceUCCEAN(filtereddata, 234, GS1Compliance)
    If (detectAllNumbersUCCEAN(replacedata234, GS1Compliance) = 0) Then
        
                        
        result = getAutoSwitchingABUCCEAN(replacedata234) 'need to update if 234
        cd = generateCheckDigitABUCCEAN(replacedata234)
        result = result + cd
        startc = 236
        stopc = 238
        result = Chr(startc) + result + Chr(stopc)
    
    Else
        
               
        cd = generateCheckDigitCUCCEAN(replacedata234)
    
        x = 0
        Skip = 0
        Do While (x < Len(replacedata234))
        
            Skip = 0
            If (Asc(Mid(replacedata234, x + 1, 1)) = 234) Then
                            
                result = result + Chr(234)
                x = x + 1
                Skip = 1
            
            End If
            
            
            
            Dim barcodechar1 As Integer
            Dim barcodechar2 As Integer
            If (Skip = 0) Then
            
                '5.7
                barcodechar1 = 0
                barcodechar2 = 0
                barcodechar1 = Asc(Mid(replacedata234, x + 1, 1))
                If (x + 1 < Len(replacedata234)) Then
                    barcodechar2 = Asc(Mid(replacedata234, x + 2, 1))
                End If

                If (barcodechar1 >= Asc("0") And barcodechar1 <= Asc("9") And barcodechar2 >= Asc("0") And barcodechar2 <= Asc("9")) Then
                
                        
                        'Existing Code
                        num = val(Mid(replacedata234, x + 1, 2)) 'Check its x+1 and not x
                        result = result + getUCCEANCCharacter(num)
                        x = x + 2
            
                
                ElseIf ((barcodechar1 >= Asc("0") And barcodechar1 <= Asc("9") And barcodechar2 = 234) Or (barcodechar1 >= Asc("0") And barcodechar1 <= Asc("9") And barcodechar2 = 0)) Then
                                
                        Dim CtoB As String
                        Dim BtoC As String
                
                        'Switch from C back to B
                        CtoB = Chr(232)
                        result = result + CtoB

                        'Single B number
                        result = result + Chr(barcodechar1)
                        x = x + 1
                                
                        'Switch from B to C
                        BtoC = Chr(231)
                        result = result + BtoC
                
                End If
            
            End If
            
        Loop
        result = result + cd
        startc = 237
        stopc = 238
        result = Chr(startc) + result + Chr(stopc)
    End If
    Encode_UCCEAN = result

End Function



Public Function replaceUCCEAN(ByVal data As String, ByVal fncvalue As Integer, ByVal GS1Compliance As Integer) As String

  result = ""
  datalength = Len(data)
  forceNULL = 0

  x = 0
  numset = 0
  exitx = 0
  
  Dim startBracketPosition(8) As Integer
  Dim stopBracketPosition(8) As Integer
  
  Dim aiset(8) As String
  Dim dataset(8) As String
   
  'Check 0 based or 1 based
  For counter = 0 To (8 - 1)
  
    startBracketPosition(counter) = -1
    stopBracketPosition(counter) = -1
    
  Next counter
  
 
  x = 0
  Do While (x < datalength)
  
    barcodecharx = Mid(data, x + 1, 1)
    If (barcodecharx = "(") Then
    
         y = x + 1
         exitx = 0
                Do While ((y < datalength) And (exitx = 0))
                
                  barcodechary = Mid(data, y + 1, 1)
                  If (barcodechary = ")") Then
                  
                      startBracketPosition(numset) = x
                      stopBracketPosition(numset) = y
                      numset = numset + 1
                      exitx = 1
                  
                  End If
                  
                  If (exitx = 0) Then
                      y = y + 1
                  End If
                  
                Loop
          x = y
     
    End If
    x = x + 1
  Loop

  If (numset = 0) Then
    
   'Mod 3.2
    'forceNULL = 1
    forceNULL = 0
    replaceUCCEAN = Chr(fncvalue) + data

  Else

      For x = 0 To numset - 1
      
          aiset(x) = ""
          dataset(x) = ""
    
          If (stopBracketPosition(x) <= startBracketPosition(x)) Then
            forceNULL = 1
            Exit For
          End If
    
          If (stopBracketPosition(x) - 1 = startBracketPosition(x)) Then
            forceNULL = 1
            Exit For
          End If
    
        For y = startBracketPosition(x) + 1 To stopBracketPosition(x) - 1
        
            barcodechar = Mid(data, y + 1, 1)
            barcodevalue = Asc(barcodechar)
            If ((barcodevalue <= Asc("9") And barcodevalue >= Asc("0")) Or (barcodevalue <= Asc("Z") And barcodevalue >= Asc("A")) Or (barcodevalue <= Asc("z") And barcodevalue >= Asc("a"))) Then
                aiset(x) = aiset(x) + barcodechar
            End If
        
        Next y
    
        If (x = numset - 1) Then
        
            For y = stopBracketPosition(x) + 1 To Len(data) - 1
            
                barcodechar = Mid(data, y + 1, 1)
                barcodevalue = Asc(barcodechar)
                If ((barcodevalue <= Asc("9") And barcodevalue >= Asc("0")) Or (barcodevalue <= Asc("Z") And barcodevalue >= Asc("A")) Or (barcodevalue <= Asc("z") And barcodevalue >= Asc("a"))) Then
                    dataset(x) = dataset(x) + barcodechar
                End If
            
            Next y
        
        Else
        
            For y = stopBracketPosition(x) + 1 To startBracketPosition(x + 1) - 1
            
                 barcodechar = Mid(data, y + 1, 1)
                 barcodevalue = Asc(barcodechar)
                If ((barcodevalue <= Asc("9") And barcodevalue >= Asc("0")) Or (barcodevalue <= Asc("Z") And barcodevalue >= Asc("A")) Or (barcodevalue <= Asc("z") And barcodevalue >= Asc("a"))) Then
                    dataset(x) = dataset(x) + barcodechar
                End If
            
            Next y
        
        End If
      Next x
    
     'After Loop Exit, forceNULL may be set to 1
     If (forceNULL = 1) Then
        
        replaceUCCEAN = ""
    
     Else
        
      result = Chr(fncvalue) + result
      fncstring = Chr(fncvalue)
      For x = 0 To numset - 1
      
        If (x = numset - 1) Then
            result = result + aiset(x) + dataset(x)
        Else
            '5.7
            If (GS1Compliance = 0) Then
                result = result + aiset(x) + dataset(x) + fncstring 'existing code
            Else
        
                'Selectively add FNC1 only if compliance is on
                If (aiset(x) = "8002" Or aiset(x) = "8003" Or aiset(x) = "8004" Or aiset(x) = "8007" Or aiset(x) = "8008" Or aiset(x) = "8020" Or aiset(x) = "240" Or aiset(x) = "241" Or aiset(x) = "250" Or aiset(x) = "251" Or aiset(x) = "400" Or aiset(x) = "401" Or aiset(x) = "403" Or aiset(x) = "420" Or aiset(x) = "421" Or aiset(x) = "423" Or aiset(x) = "10" Or aiset(x) = "21" Or aiset(x) = "22" Or aiset(x) = "23" Or aiset(x) = "30" Or aiset(x) = "37" Or aiset(x) = "90" Or aiset(x) = "91" Or aiset(x) = "92" Or aiset(x) = "93" Or aiset(x) = "94" Or aiset(x) = "95" Or aiset(x) = "96" Or aiset(x) = "97" Or aiset(x) = "98" Or aiset(x) = "99") Then
                    result = result + aiset(x) + dataset(x) + fncstring
                Else
                    result = result + aiset(x) + dataset(x)
                End If
                    
        
            End If

        End If
      
      Next x
      replaceUCCEAN = result
      
      End If
  
  End If
  
  If (forceNULL = 1) Then
    replaceUCCEAN = ""
  End If

End Function
   


Public Function detectAllNumbersUCCEAN(ByVal data As String, ByVal GS1Compliance As Integer) As Integer

  'all numbers and even digits
  result = ""
  allnumbers = 1
  numNumbers = 0
  datalength = Len(data)


  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = Asc(barcodechar)
    
    If ((barcodevalue <= Asc("9")) And (barcodevalue >= Asc("0"))) Then
        numNumbers = numNumbers + 1
    ElseIf (barcodevalue = 234) Then
                
        '5.7 Loosen check when compliance is 1
        If (GS1Compliance = 0) Then
        
            If ((numNumbers Mod 2) = 1) Then
                allnumbers = 0
            End If
            
        End If
        numNumbers = 0
    Else
 
        allnumbers = 0
    End If
  Next x
  
  '5.7 Loosen check when compliance is 1
  If (GS1Compliance = 0) Then
    If ((numNumbers Mod 2) = 1) Then
          allnumbers = 0
    End If
  End If
  
  detectAllNumbersUCCEAN = allnumbers

End Function



Public Function getAutoSwitchingABUCCEAN(ByVal data As String) As String

  Dim x As Integer
  
  datalength = 0
  strResult = ""
  shiftchar = Chr(230)

  datalength = Len(data)

  x = 0
  Do While (x < Len(data))

    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = Asc(barcodechar)

    If ((barcodevalue <= 31) And (barcodevalue >= 0)) Then
            
        strResult = strResult + shiftchar
        barcodevalue = barcodevalue + 96
        strResult = strResult + Chr(barcodevalue)
    
    ElseIf (barcodevalue = 127) Then
    
        barcodevalue = barcodevalue + 100
        strResult = strResult + Chr(barcodevalue)
    
    Else
    
        num = ScanAhead_8orMore_NumbersUCCEAN(data, x)
        If (num >= 8) Then
         
            
            strResult = OptimizeNumbersUCCEAN(data, x, strResult, num)
            
            x = x - 1
        
        Else
            
            strResult = strResult + barcodechar
        End If
    
    End If

    x = x + 1
  Loop
  
  getAutoSwitchingABUCCEAN = strResult


End Function



Public Function generateCheckDigitABUCCEAN(ByVal data) As String


  datalength = 0
  sum = 104
  result = -1
  strResult = ""
 
  datalength = Len(data)

  x = 0
  num = 0
  weight = 1
  Do While (x < Len(data))
  
    num = ScanAhead_8orMore_NumbersUCCEAN(data, x)
    If (num >= 8) Then
    
       endpoint = x + num

      '105 to Switch from B to C
      BtoC = 99
      sum = sum + (BtoC * (weight))
      weight = weight + 1

      Do While (x < endpoint)
      
        num = val(Mid(data, x + 1, 2))
        sum = sum + (num * (weight))
        x = x + 2
        weight = weight + 1
      Loop

      '104 to Switch from C back to B
      CtoB = 100
      sum = sum + (CtoB * (weight))
      weight = weight + 1

    
    Else
    
        If (Asc(Mid(data, x + 1, 1)) = 234) Then
        
            num = 102
            sum = sum + (num * (weight))
        
        Else
        
            num = Asc(Mid(data, x + 1, 1))
            sum = sum + getUCCEANABValue(num) * (weight)
        
        End If
        x = x + 1
        weight = weight + 1
    End If
  Loop

  result = sum Mod 103
  strResult = getUCCEANABCharacter(result)

  generateCheckDigitABUCCEAN = strResult

End Function


Public Function generateCheckDigitCUCCEAN(ByVal data As String) As String

  datalength = 0
  sum = 105
  num = 0
  result = -1
  strResult = ""

  datalength = Len(data)

  x = 0
  weight = 1
  Skip = 0
  Do While (x < Len(data))
  
    Skip = 0
    If (Asc(Mid(data, x + 1, 1)) = 234) Then
        num = 102
        sum = sum + (num * (weight))
        x = x + 1
        weight = weight + 1
        Skip = 1
        
    End If


            '''''''''''''''''''''''
            Dim barcodechar1 As Integer
            Dim barcodechar2 As Integer
            If (Skip = 0) Then
            
                '5.7
                barcodechar1 = 0
                barcodechar2 = 0
                barcodechar1 = Asc(Mid(data, x + 1, 1))
                If (x + 1 < Len(data)) Then
                    barcodechar2 = Asc(Mid(data, x + 2, 1))
                End If

                If (barcodechar1 >= Asc("0") And barcodechar1 <= Asc("9") And barcodechar2 >= Asc("0") And barcodechar2 <= Asc("9")) Then
                
                        
                        'Existing Code
                        num = val(Mid(data, x + 1, 2)) 'Check its x+1 and not x
                        sum = sum + (num * (weight))
                        x = x + 2
                        weight = weight + 1
            
                
                ElseIf ((barcodechar1 >= Asc("0") And barcodechar1 <= Asc("9") And barcodechar2 = 234) Or (barcodechar1 >= Asc("0") And barcodechar1 <= Asc("9") And barcodechar2 = 0)) Then
                                
                        Dim CtoB As Integer
                        Dim BtoC As Integer
                
                        'Swing to B
                        CtoB = 100
                        sum = sum + (CtoB * (weight))
                        weight = weight + 1


                        'Single B number
                        num = Asc(Mid(data, x + 1, 1))
                        sum = sum + getUCCEANABValue(num) * (weight)
                        weight = weight + 1
                        
                        
                                
                        'Swing back to C
                        BtoC = 99
                        sum = sum + (BtoC * (weight))
                        weight = weight + 1
                        x = x + 1
                
                End If
            
            End If
            '''''''''''''''''''''''

  Loop

  result = sum Mod 103
  strResult = getUCCEANCCharacter(result)

  generateCheckDigitCUCCEAN = strResult

End Function

Public Function getUCCEANCCharacter(ByVal inputvalue As Integer) As String


    If (inputvalue <= 94) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 106) And (inputvalue >= 95) Then
        'Mod 3.2
        'inputvalue = inputvalue + 32 + 101
      inputvalue = inputvalue + 32 + 100
    Else
        inputvalue = -1
    End If

    getUCCEANCCharacter = Chr(inputvalue)

End Function


Public Function getUCCEANABCharacter(ByVal inputvalue As Integer) As String

    If (inputvalue <= 94) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 106) And (inputvalue >= 95) Then
        inputvalue = inputvalue + 100 + 32
    Else
        inputvalue = -1
     End If

    getUCCEANABCharacter = Chr(inputvalue)

End Function


Public Function getUCCEANABValue(ByVal inputchar As Integer) As Integer

    returnvalue = 0

    If (inputchar <= 31) And (inputchar >= 0) Then
        returnvalue = (inputchar + 64)
    ElseIf ((inputchar <= 127) And (inputchar >= 32)) Then
        returnvalue = (inputchar - 32)
    ElseIf (inputchar = 230) Then
        returnvalue = 98  'Shift
    Else
        returnvalue = -1
    End If

    getUCCEANABValue = returnvalue

End Function


Public Function filterInputUCCEAN(ByVal data As String) As String

  result = ""
  datalength = Len(data)

  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = Asc(barcodechar)

    If ((barcodevalue <= 127) And (barcodevalue >= 0)) Then
        result = result + barcodechar
    End If
  
  Next x

  filterInputUCCEAN = result

End Function

Public Function OptimizeNumbersUCCEAN(ByVal data As String, x As Integer, ByVal strResult As String, ByVal num As Integer) As String


        '105 to Switch from B to C
        BtoC = Chr(231)
        strResult = strResult + BtoC
                    
        endpoint = x + num
        Do While (x < endpoint)
        
            twonum = val(Mid(data, x + 1, 2))
            
            strResult = strResult + getUCCEANCCharacter(twonum)
            x = x + 2
        
        Loop

        '104 to Switch from C back to B
        CtoB = Chr(232)
        strResult = strResult + CtoB
        
        OptimizeNumbersUCCEAN = strResult

End Function



Public Function ScanAhead_8orMore_NumbersUCCEAN(ByVal data As String, ByVal x As Integer) As Integer

    numNumbers = 0
    exitx = 0
    Do While ((x < Len(data)) And (exitx = 0))
        barcodechar = Mid(data, x + 1, 1)
        barcodevalue = Asc(barcodechar)
        If ((barcodevalue >= Asc("0")) And (barcodevalue <= Asc("9"))) Then
            numNumbers = numNumbers + 1
        Else
            exitx = 1
        End If
        x = x + 1
    Loop

    'return even digits
    If (numNumbers > 8) Then
        If ((numNumbers Mod 2) = 1) Then
            numNumbers = numNumbers - 1
        End If
    End If
    ScanAhead_8orMore_NumbersUCCEAN = numNumbers

End Function

Function MOD_10(ByVal data As String) As String

    Dim result As String
    Dim multi As Integer
    Dim total As Integer
    Dim cv As Integer
    
    If (Len(data) = 0) Then
        result = ""
    Else
    cv = 0
    multi = 3
    total = 0
    result = ""
    
    For x = 1 To Len(data)
        If (Asc(Mid(data, x, 1)) - 48 <= 9 And Asc(Mid(data, x, 1)) - 48 >= 0) Then
            result = result + Mid(data, x, 1)
        End If
    Next x

    For x = 1 To Len(result)
        cv = Asc(Mid(result, x, 1)) - 48
        total = total + cv * multi
        If (multi = 3) Then
            multi = 1
        Else
            multi = 3
        End If
            
    Next x
    
    Dim modresult As Integer
    modresult = total Mod 10
    If (modresult <> 0) Then
        modresult = 10 - modresult
    End If
    
    result = CStr(modresult)
    End If
    
    MOD_10 = result

End Function
 
Function SSCC(ByVal data As String) As String
    SSCC = SSCC_PAD(data) + MOD_10(SSCC_PAD(data))
End Function

Function GTIN(ByVal data As String) As String
    GTIN = GTIN_PAD(data) + MOD_10(GTIN_PAD(data))
End Function

 
Function SSCC_PAD(ByVal data As String) As String
    Dim result As String
    For x = 1 To Len(data)
       If (Asc(Mid(data, x, 1)) - 48 <= 9 And Asc(Mid(data, x, 1)) - 48 >= 0) Then
           result = result + Mid(data, x, 1)
       End If
    Next x
    
    If (Len(result) = 0) Then
        result = ""
    Else
        If (Len(result) > 17) Then
            result = left(result, 17)
        End If
        If (Len(result) < 17) Then
            For x = 1 To (17 - Len(result))
                result = "0" + result
            Next x
        End If
    End If
    SSCC_PAD = result
End Function

Function GTIN_PAD(ByVal data As String) As String
    Dim result As String
    For x = 1 To Len(data)
       If (Asc(Mid(data, x, 1)) - 48 <= 9 And Asc(Mid(data, x, 1)) - 48 >= 0) Then
           result = result + Mid(data, x, 1)
       End If
    Next x
    If (Len(result) = 0) Then
        result = ""
    Else
        If (Len(result) > 13) Then
            result = left(result, 13)
        End If
        If (Len(result) < 13) Then
            For x = 1 To (13 - Len(result))
                result = "0" + result
            Next x
        End If
    End If
    GTIN_PAD = result
End Function

'==========================================================================
'Encode_ITF14
'==========================================================================

Public Function filterInput_ITF14(ByVal data As String) As String

    result = ""
    datalength = Len(data)
   
    barcodechar = ""
    For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    
    If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
         result = result + barcodechar
    End If
    Next x
  
    filterInput_ITF14 = result

End Function

Public Function getITF14Character(ByVal inputvalue As Integer) As String

    If (inputvalue <= 90) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 99) And (inputvalue >= 91) Then
        inputvalue = inputvalue + 100
    Else
        inputvalue = -1
    End If

    getITF14Character = Chr(inputvalue)

End Function

Public Function generateCheckDigit_ITF14(ByVal data As String) As String

  datalength = 0
  lastcharpos = 0
  result = 0
  strResult = ""
  barcodechar = ""
  barcodevalue = 0
  toggle = 1
  sum = 0


  datalength = Len(data)
  lastcharpos = datalength - 1

  For x = lastcharpos To 0 Step -1
  
      barcodechar = Mid(data, x + 1, 1)
      barcodevalue = (AscW(barcodechar) - 48)
  
        
      If toggle = 1 Then
            sum = sum + (barcodevalue * 3)
            toggle = 0
      Else
            sum = sum + barcodevalue
            toggle = 1
      End If
  Next x

  If ((sum Mod 10) = 0) Then
        result = AscW("0")
  Else
        result = (10 - (sum Mod 10)) + AscW("0")
  End If

  strResult = Chr(result)
  generateCheckDigit_ITF14 = strResult


End Function


Public Function Encode_ITF14(ByVal data As String, ByVal chk As Integer, ByVal itfrectangle As Integer) As String

    Dim result As String
    Dim cd As String
    Dim filtereddata As String
          
    transformdata = ""
    filtereddata = filterInput_ITF14(data)
    filteredlength = Len(filtereddata)
    
    'Input String including check digit must be even digit numbers in ITF14
    If chk = 1 Then

        If filteredlength > 253 Then
            filtereddata = left$(filtereddata, 253)
            
        End If

        If (Len(filtereddata) Mod 2 = 0) Then   'Make it odd number of digits
            filtereddata = "0" + filtereddata
        End If

        cd = generateCheckDigit_ITF14(filtereddata)  'Together with check digit, it is even number of digits
    
    Else
        
        If filteredlength > 254 Then
            filtereddata = left$(filtereddata, 254)
        End If

        If (Len(filtereddata) Mod 2 = 1) Then 'Make it even number of digits
            filtereddata = "0" + filtereddata
         End If
    
    End If
    
    
    filtereddata = filtereddata + cd

    
    num = 0
    For x = 0 To Len(filtereddata) - 1 Step 2
    
        num = val(Mid(filtereddata, x + 1, 2))
        result = result + getITF14Character(num)
    
    Next x
    
    'ITF14
    If (itfrectangle = 1) Then
        
        startITF = Chr(124)
        stopITF = Chr(126)
        result = startITF + result + stopITF
    
    Else
        result = "{" + result + "}"
    End If
        
    Encode_ITF14 = result

End Function

'======================================================================================================
'Encode_Code128Auto
'======================================================================================================

Public Function filterInput_Code128Auto(ByVal data As String) As String

    result = ""
    datalength = Len(data)
  
    barcodechar = ""
    For x = 0 To datalength - 1
  
        barcodechar = Mid(data, x + 1, 1)
        barcodevalue = Asc(barcodechar)
    
    If (barcodevalue <= 127) And (barcodevalue >= 0) Then
        result = result + barcodechar
    End If
    
    Next x
  
    filterInput_Code128Auto = result

End Function



Public Function getCode128ABValueAuto(ByVal inputchar As Integer) As Integer


    returnvalue = 0

    If (inputchar <= 31) And (inputchar >= 0) Then
        returnvalue = (inputchar + 64)
    ElseIf (inputchar <= 127) And (inputchar >= 32) Then
        returnvalue = (inputchar - 32)
    ElseIf (inputchar = 230) Then
        returnvalue = 98 'Shift
    Else
        returnvalue = -1
    End If

    getCode128ABValueAuto = returnvalue
 
End Function


Public Function getCode128ABCharacterAuto(ByVal inputvalue As Integer) As String

    If (inputvalue <= 94) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 106) And (inputvalue >= 95) Then
        inputvalue = inputvalue + 100 + 32
    Else
        inputvalue = -1
    End If

    getCode128ABCharacterAuto = Chr(inputvalue)


End Function


Public Function getCode128CCharacterAuto(ByVal inputvalue As Integer) As String


    If (inputvalue <= 94) And (inputvalue >= 0) Then
        inputvalue = inputvalue + 32
    ElseIf (inputvalue <= 106) And (inputvalue >= 95) Then
        inputvalue = inputvalue + 32 + 100
    Else
        inputvalue = -1
    End If
    
    getCode128CCharacterAuto = Chr(inputvalue)

End Function

Public Function generateCheckDigit_Code128ABAuto(ByVal data As String) As String
  
  datalength = 0
  sum = 104
  result = -1
  strResult = ""

  datalength = Len(data)
  
  num = 0
  weight = 1
  
  x = 0
  Do While (x < Len(data))
  'For x = 0 To Len(data) - 1
  
    num = ScanAhead_8orMore_Numbers(data, x)
    If (num >= 8) Then
    
      endpoint = x + num

      '105 to Switch from B to C
      BtoC = 99
      sum = sum + (BtoC * (weight))
      weight = weight + 1
      
      'For z = x To endpoint - 1 Step 2
      Do While (x < endpoint)
      
        num = val(Mid(data, x + 1, 2))
        sum = sum + (num * (weight))
        x = x + 2
        weight = weight + 1

      Loop
      'Next z
      'x = z

      '104 to Switch from C back to B
      CtoB = 100
      sum = sum + (CtoB * (weight))
      weight = weight + 1
    
    
    Else
    
        num = Asc(Mid$(data, x + 1, 1))
        sum = sum + (getCode128ABValueAuto(num) * (weight))
        x = x + 1
        weight = weight + 1
    
    End If
  
  'Next x
  Loop

  result = sum Mod 103
  strResult = getCode128ABCharacterAuto(result)
  generateCheckDigit_Code128ABAuto = strResult
  

End Function



Public Function generateCheckDigit_Code128CAuto(ByVal data As String) As String
    
  datalength = 0
  sum = 105
  result = -1
  strResult = ""

  datalength = Len(data)

  
  num = 0
  weight = 1
  
  For x = 0 To Len(data) - 1 Step 2
  
    num = val(Mid$(data, x + 1, 2))
    sum = sum + (num * weight)
    weight = weight + 1
  
  Next x

  result = sum Mod 103
  strResult = getCode128CCharacterAuto(result)

  generateCheckDigit_Code128CAuto = strResult

End Function

Public Function getAutoSwitchingAB(ByVal data As String) As String


  datalength = 0
  strResult = ""
  shiftchar = Chr(230)

  datalength = Len(data)
  Dim barcodechar As String
  Dim x As Integer

  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = Asc(barcodechar)

    If (barcodevalue = 31) Then
        
        barcodechar = Chr(Asc(barcodechar) + 96 + 100)
        strResult = strResult + barcodechar
    
    ElseIf (barcodevalue = 127) Then
    
        barcodechar = Chr(Asc(barcodechar) + 100)
        strResult = strResult + barcodechar
    
    Else
    
        num = ScanAhead_8orMore_Numbers(data, x)
        If (num >= 8) Then
            
            strResult = OptimizeNumbers(data, x, strResult, num)
            x = x - 1
        
        Else
            strResult = strResult + barcodechar
        End If
            
        
    End If
 
    
  Next x
    

  getAutoSwitchingAB = strResult

 End Function


Public Function detectAllNumbers(ByVal data As String) As Integer


  'all numbers and even digits
  result = ""
  allnumbers = 1
  
  datalength = Len(data)

  If (datalength Mod 2 = 1) Then
     allnumbers = 0
  Else
    
        For x = 0 To datalength - 1
            barcodechar = Asc(Mid(data, x + 1, 1))
            If (barcodechar <= Asc("9")) And (barcodechar >= Asc("0")) Then
                'do nothing
            Else
                allnumbers = 0
            End If
        Next x
  End If

  detectAllNumbers = allnumbers

End Function


Public Function addShift(ByVal data As String) As String

  datalength = 0
  strResult = ""
  shiftchar = Chr(230)

  datalength = Len(data)
  
  For x = 0 To datalength - 1
      
    barcodechar = Mid(data, x + 1, 1)
    barcodevalue = Asc(barcodechar)

    'Code 128 A - 0..31 Add Shift
    If (barcodevalue <= 31) And (barcodevalue >= 0) Then
    
        
        strResult = strResult + shiftchar
        barcodechar = Chr(Asc(barcodechar) + 96)
        strResult = strResult + barcodechar
    
    Else
        strResult = strResult + barcodechar
    End If

    
  Next x
  
  addShift = strResult


End Function



Public Function Encode_Code128Auto(ByVal data As String) As String

 
    cd = ""
    result = ""
    shiftdata = ""
    
    filtereddata = filterInput_Code128Auto(data)
    
    filteredlength = Len(filtereddata)
    If (filteredlength > 254) Then
        filtereddata = left$(filtereddata, 254)
    End If
    
        
    If (detectAllNumbers(filtereddata) = 0) Then
            
        filtereddata = addShift(filtereddata)
        cd = generateCheckDigit_Code128ABAuto(filtereddata)
        'Convert Ascii 31 / Ascii 127 to 227
        filtereddata = getAutoSwitchingAB(filtereddata)

        
        'MsgBox Asc(cd)
        
        filtereddata = filtereddata + cd
        result = filtereddata
        
        startc = 236
        stopc = 238
        result = Chr(startc) + result + Chr(stopc)
    
    Else
        
        
        cd = generateCheckDigit_Code128CAuto(filtereddata)
        lenFiltered = Len(filtereddata)
        
        For x = 0 To (lenFiltered - 1) Step 2
        
            tstr = Mid$(filtereddata, x + 1, 2)
            num = val(tstr)
            result = result + getCode128CCharacterAuto(num)
        
        Next x
        
        
        result = result + cd
        startc = 237
        stopc = 238
        result = Chr(startc) + result + Chr(stopc)
        
    End If

    Encode_Code128Auto = result

 End Function



Public Function OptimizeNumbers(ByVal data As String, x As Integer, ByVal strResult As String, ByVal num As Integer) As String

        '105 to Switch from B to C
        BtoC = Chr(231)
        strResult = strResult + BtoC
                    
        endpoint = x + num
        Do While (x < endpoint)
                
            twonum = val(Mid(data, x + 1, 2))
            strResult = strResult + getCode128CCharacterAuto(twonum)
            x = x + 2
            
        Loop

        '104 to Switch from C back to B
        CtoB = Chr(232)
        strResult = strResult + CtoB
        
        OptimizeNumbers = strResult

End Function

Public Function ScanAhead_8orMore_Numbers(ByVal data As String, ByVal x As Integer) As Integer

    numNumbers = 0
    exitx = 0
    Do While ((x < Len(data)) And (exitx = 0))
    
        barcodechar = Mid(data, x + 1, 1)
        barcodevalue = Asc(barcodechar)
        If (barcodevalue >= Asc("0") And barcodevalue <= Asc("9")) Then
            numNumbers = numNumbers + 1
        Else
            exitx = 1
        End If
        x = x + 1
        
    Loop

    'return even digits
    If (numNumbers > 8) Then
        If (numNumbers Mod 2 = 1) Then
            numNumbers = numNumbers - 1
        End If
    End If
    ScanAhead_8orMore_Numbers = numNumbers

End Function

'======================================================================================================
'Industrial2of5
'======================================================================================================

Public Function Encode_Industrial2of5(ByVal data As String, Optional ByVal chk As Integer = 0) As String

    Dim result As String
    Dim cd As String
    Dim filtereddata As String

    cd = ""
    result = ""
    filtereddata = filterInput_Industrial2of5(data)
    filteredlength = Len(filtereddata)
    
    
    If chk = 1 Then
    
        If filteredlength > 254 Then
            filtereddata = left$(filtereddata, 254)
        End If
        cd = generateCheckDigit_Industrial2of5(filtereddata)
    
    Else
    
        If filteredlength > 255 Then
            filtereddata = left$(filtereddata, 255)
        End If
    
    End If

    filtereddata = filtereddata + cd
    result = "{" + filtereddata + "}"
    

    Encode_Industrial2of5 = result

End Function



Public Function filterInput_Industrial2of5(ByVal data As String) As String

    result = ""
    datalength = Len(data)
   
    barcodechar = ""
    For x = 0 To datalength - 1
  
        barcodechar = Mid(data, x + 1, 1)
        If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
             result = result + barcodechar
        End If
    Next x
  
    filterInput_Industrial2of5 = result

End Function



Public Function generateCheckDigit_Industrial2of5(ByVal data As String) As String

  datalength = 0
  lastcharpos = 0
  result = 0
  strResult = ""
  barcodechar = ""
  barcodevalue = 0
  toggle = 1
  sum = 0


  datalength = Len(data)
  lastcharpos = datalength - 1

  For x = lastcharpos To 0 Step -1
  
      barcodechar = Mid(data, x + 1, 1)
      barcodevalue = (AscW(barcodechar) - 48)
  
        
      If toggle = 1 Then
            sum = sum + (barcodevalue * 3)
            toggle = 0
      Else
            sum = sum + barcodevalue
            toggle = 1
      End If
  Next x

  If ((sum Mod 10) = 0) Then
        result = AscW("0")
  Else
        result = (10 - (sum Mod 10)) + AscW("0")
  End If

  strResult = Chr(result)
  generateCheckDigit_Industrial2of5 = strResult


End Function

'======================================================================================================
'ModifiedPlessy
'======================================================================================================

Public Function Encode_ModifiedPlessy(ByVal data As String, Optional ByVal chk As Integer = 0) As String


    Dim result As String
    Dim cd As String
    Dim filtereddata As String

    cd = ""
    result = ""
    filtereddata = filterInput_ModifiedPlessy(data)
    filteredlength = Len(filtereddata)
    
    
    If chk = 1 Then
    
        If filteredlength > 15 Then
            filtereddata = left$(filtereddata, 15)
        End If
        cd = generateCheckDigit_ModifiedPlessy(filtereddata)
    
    Else
    
        If filteredlength > 16 Then
            filtereddata = left$(filtereddata, 16)
        End If
    
    End If

    filtereddata = filtereddata + cd
    result = "{" + filtereddata + "}"

    Encode_ModifiedPlessy = result

End Function


Public Function generateCheckDigit_ModifiedPlessy(ByVal data As String) As String


  doublechar = ""
  doubleStr = ""
  doubleNumber = 0

  datalength = 0
  lastcharpos = 0
  result = 0
  strResult = ""
  barcodechar = ""
  barcodevalue = 0
  toggle = 1
  sum = 0


  datalength = Len(data)
  lastcharpos = datalength - 1

  '=================================

  For x = lastcharpos To 0 Step -1
  
      barcodechar = Mid(data, x + 1, 1)
      barcodevalue = (AscW(barcodechar) - 48)
  
        
      If toggle = 1 Then
            doubleStr = barcodechar + doubleStr
            toggle = 0
      Else
            sum = sum + barcodevalue
            toggle = 1
      End If
  Next x
  
  
  doubleNumber = val(doubleStr)
  doubleNumber = doubleNumber * 2
  
  doubleStr = Str$(doubleNumber)
  doubleStr = Trim(doubleStr)
  
  For y = 0 To Len(doubleStr) - 1
  
    barcodechar = Mid(doubleStr, y + 1, 1)
    barcodevalue = (AscW(barcodechar) - 48)
    sum = sum + barcodevalue
    
    
  Next y
  
  
  '=================================

  If ((sum Mod 10) = 0) Then
        result = AscW("0")
  Else
        result = (10 - (sum Mod 10)) + AscW("0")
  End If

  strResult = Chr(result)
  generateCheckDigit_ModifiedPlessy = strResult


End Function


Public Function filterInput_ModifiedPlessy(ByVal data As String) As String

    result = ""
    datalength = Len(data)
   
    barcodechar = ""
    For x = 0 To datalength - 1
    
  
        barcodechar = Mid(data, x + 1, 1)
        If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
             result = result + barcodechar
        End If
    Next x
  
    filterInput_ModifiedPlessy = result

End Function

'======================================================================================================
'Code93
'======================================================================================================
Public Function Encode_Code93(ByVal data As String, Optional ByVal chk As Integer = 1) As String

      
    Dim result As String
    Dim cd As String
    Dim filtereddata As String
    Dim datalength As Integer
    Dim barcodevalue As Integer
    
    cd = ""
    result = ""
    filtereddata = filterInput_Code93(data)
    filteredlength = Len(filtereddata)

    If chk = 1 Then
    
        If filteredlength > 254 Then
        
            filtereddata = left$(filtereddata, 254)
            
        End If
        cd = generateCheckDigit_Code93(filtereddata)
    
    Else
    
        If filteredlength > 255 Then
            
            filtereddata = left$(filtereddata, 255)
            
        End If
    
    End If
    
    
    datalength = 0
    datalength = Len(filtereddata)
    For x = 0 To datalength - 1
        
        barcodevalue = Asc(Mid(filtereddata, x + 1, 1))
        result = result + translate_Code93(barcodevalue)
    
    Next x
    

    Encode_Code93 = Chr(197) + result + cd + Chr(198)
    
    

End Function




Public Function filterInput_Code93(ByVal data As String) As String

  
  Dim result As String
  Dim barcodechar As Integer
  result = ""
  datalength = Len(data)
  
  For x = 0 To datalength - 1
        barcodechar = Asc(Mid(data, x + 1, 1))
        If (barcodechar >= 0) And (barcodechar <= 127) Then
            result = result & Chr(barcodechar)
        End If
  Next x

  filterInput_Code93 = result

End Function


Public Function translate_Code93(ByVal inputx As Integer) As String

    result = ""
    Select Case (inputx)
    
        Case 0:
            result = result + Chr(194)
            result = result + Chr(85)
            
        Case 1:
            result = result + Chr(193)
            result = result + Chr(65)
            
        Case 2:
            result = result + Chr(193)
            result = result + Chr(66)
            
        Case 3:
            result = result + Chr(193)
            result = result + Chr(67)
            
        Case 4:
            result = result + Chr(193)
            result = result + Chr(68)
            
        Case 5:
            result = result + Chr(193)
            result = result + Chr(69)
            
        Case 6:
            result = result + Chr(193)
            result = result + Chr(70)
            
        Case 7:
            result = result + Chr(193)
            result = result + Chr(71)
            
        Case 8:
            result = result + Chr(193)
            result = result + Chr(72)
            
        Case 9:
            result = result + Chr(193)
            result = result + Chr(73)
            
        Case 10:
            result = result + Chr(193)
            result = result + Chr(74)
            
        Case 11:
            result = result + Chr(193)
            result = result + Chr(75)
            
        Case 12:
            result = result + Chr(193)
            result = result + Chr(76)
            
        Case 13:
            result = result + Chr(193)
            result = result + Chr(77)
            
        Case 14:
            result = result + Chr(193)
            result = result + Chr(78)
            
        Case 15:
            result = result + Chr(193)
            result = result + Chr(79)
            
        Case 16:
            result = result + Chr(193)
            result = result + Chr(80)
            
        Case 17:
            result = result + Chr(193)
            result = result + Chr(81)
            
        Case 18:
            result = result + Chr(193)
            result = result + Chr(82)
            
        Case 19:
            result = result + Chr(193)
            result = result + Chr(83)
            
        Case 20:
            result = result + Chr(193)
            result = result + Chr(84)
            
        Case 21:
            result = result + Chr(193)
            result = result + Chr(85)
            
        Case 22:
            result = result + Chr(193)
            result = result + Chr(86)
            
        Case 23:
            result = result + Chr(193)
            result = result + Chr(87)
            
        Case 24:
            result = result + Chr(193)
            result = result + Chr(88)
            
        Case 25:
            result = result + Chr(193)
            result = result + Chr(89)
            
        Case 26:
            result = result + Chr(193)
            result = result + Chr(90)
            
        Case 27:
            result = result + Chr(194)
            result = result + Chr(65)
            
        Case 28:
            result = result + Chr(194)
            result = result + Chr(66)
            
        Case 29:
            result = result + Chr(194)
            result = result + Chr(67)
            
        Case 30:
            result = result + Chr(194)
            result = result + Chr(68)
            
        Case 31:
            result = result + Chr(194)
            result = result + Chr(69)
            
        Case 32:
            result = result + Chr(32)
            
        Case 33:
            result = result + Chr(195)
            result = result + Chr(65)
            
        Case 34:
            result = result + Chr(195)
            result = result + Chr(66)
            
        Case 35:
            result = result + Chr(195)
            result = result + Chr(67)
            
        Case 36:
            result = result + Chr(36)
            
        Case 37:
            result = result + Chr(37)
            
        Case 38:
            result = result + Chr(195)
            result = result + Chr(70)
            
        Case 39:
            result = result + Chr(195)
            result = result + Chr(71)
            
        Case 40:
            result = result + Chr(195)
            result = result + Chr(72)
            
        Case 41:
            result = result + Chr(195)
            result = result + Chr(73)
            
        Case 42:
            result = result + Chr(195)
            result = result + Chr(74)
            
        Case 43:
            result = result + Chr(43)
            
        Case 44:
            result = result + Chr(195)
            result = result + Chr(76)
            
        Case 45:
            result = result + Chr(45)
            
        Case 46:
            result = result + Chr(46)
            
        Case 47:
            result = result + Chr(47)
            
        Case 48:
            result = result + Chr(48)
            
        Case 49:
            result = result + Chr(49)
            
        Case 50:
            result = result + Chr(50)
            
        Case 51:
            result = result + Chr(51)
            
        Case 52:
            result = result + Chr(52)
            
        Case 53:
            result = result + Chr(53)
            
        Case 54:
            result = result + Chr(54)
            
        Case 55:
            result = result + Chr(55)
            
        Case 56:
            result = result + Chr(56)
            
        Case 57:
            result = result + Chr(57)
            
        Case 58:
            result = result + Chr(195)
            result = result + Chr(90)
            
        Case 59:
            result = result + Chr(194)
            result = result + Chr(70)
            
        Case 60:
            result = result + Chr(194)
            result = result + Chr(71)
            
        Case 61:
            result = result + Chr(194)
            result = result + Chr(72)
            
        Case 62:
            result = result + Chr(194)
            result = result + Chr(73)
            
        Case 63:
            result = result + Chr(194)
            result = result + Chr(74)
            
        Case 64:
            result = result + Chr(194)
            result = result + Chr(86)
            
        Case 65:
            result = result + Chr(65)
            
        Case 66:
            result = result + Chr(66)
            
        Case 67:
            result = result + Chr(67)
            
        Case 68:
            result = result + Chr(68)
            
        Case 69:
            result = result + Chr(69)
            
        Case 70:
            result = result + Chr(70)
            
        Case 71:
            result = result + Chr(71)
            
        Case 72:
            result = result + Chr(72)
            
        Case 73:
            result = result + Chr(73)
            
        Case 74:
            result = result + Chr(74)
            
        Case 75:
            result = result + Chr(75)
            
        Case 76:
            result = result + Chr(76)
            
        Case 77:
            result = result + Chr(77)
            
        Case 78:
            result = result + Chr(78)
            
        Case 79:
            result = result + Chr(79)
            
        Case 80:
            result = result + Chr(80)
            
        Case 81:
            result = result + Chr(81)
            
        Case 82:
            result = result + Chr(82)
            
        Case 83:
            result = result + Chr(83)
            
        Case 84:
            result = result + Chr(84)
            
        Case 85:
            result = result + Chr(85)
            
        Case 86:
            result = result + Chr(86)
            
        Case 87:
            result = result + Chr(87)
            
        Case 88:
            result = result + Chr(88)
            
        Case 89:
            result = result + Chr(89)
            
        Case 90:
            result = result + Chr(90)
            
        Case 91:
            result = result + Chr(194)
            result = result + Chr(75)
            
        Case 92:
            result = result + Chr(194)
            result = result + Chr(76)
            
        Case 93:
            result = result + Chr(194)
            result = result + Chr(77)
            
        Case 94:
            result = result + Chr(194)
            result = result + Chr(78)
            
        Case 95:
            result = result + Chr(194)
            result = result + Chr(79)
            
        Case 96:
            result = result + Chr(194)
            result = result + Chr(87)
            
        Case 97:
            result = result + Chr(196)
            result = result + Chr(65)
            
        Case 98:
            result = result + Chr(196)
            result = result + Chr(66)
            
        Case 99:
            result = result + Chr(196)
            result = result + Chr(67)
            
        Case 100:
            result = result + Chr(196)
            result = result + Chr(68)
            
        Case 101:
            result = result + Chr(196)
            result = result + Chr(69)
            
        Case 102:
            result = result + Chr(196)
            result = result + Chr(70)
            
        Case 103:
            result = result + Chr(196)
            result = result + Chr(71)
            
        Case 104:
            result = result + Chr(196)
            result = result + Chr(72)
            
        Case 105:
            result = result + Chr(196)
            result = result + Chr(73)
            
        Case 106:
            result = result + Chr(196)
            result = result + Chr(74)
            
        Case 107:
            result = result + Chr(196)
            result = result + Chr(75)
            
        Case 108:
            result = result + Chr(196)
            result = result + Chr(76)
            
        Case 109:
            result = result + Chr(196)
            result = result + Chr(77)
            
        Case 110:
            result = result + Chr(196)
            result = result + Chr(78)
            
        Case 111:
            result = result + Chr(196)
            result = result + Chr(79)
            
        Case 112:
            result = result + Chr(196)
            result = result + Chr(80)
            
        Case 113:
            result = result + Chr(196)
            result = result + Chr(81)
            
        Case 114:
            result = result + Chr(196)
            result = result + Chr(82)
            
        Case 115:
            result = result + Chr(196)
            result = result + Chr(83)
            
        Case 116:
            result = result + Chr(196)
            result = result + Chr(84)
            
        Case 117:
            result = result + Chr(196)
            result = result + Chr(85)
            
        Case 118:
            result = result + Chr(196)
            result = result + Chr(86)
            
        Case 119:
            result = result + Chr(196)
            result = result + Chr(87)
            
        Case 120:
            result = result + Chr(196)
            result = result + Chr(88)
            
        Case 121:
            result = result + Chr(196)
            result = result + Chr(89)
            
        Case 122:
            result = result + Chr(196)
            result = result + Chr(90)
            
        Case 123:
            result = result + Chr(194)
            result = result + Chr(80)
            
        Case 124:
            result = result + Chr(194)
            result = result + Chr(81)
            
        Case 125:
            result = result + Chr(194)
            result = result + Chr(82)
            
        Case 126:
            result = result + Chr(194)
            result = result + Chr(83)
            
        Case 127:
            result = result + Chr(194)
            result = result + Chr(84)
            
    End Select
    translate_Code93 = result

End Function



Public Function getCode93Value(ByVal inputx As Integer) As Integer

    If (inputx >= 193) And (inputx <= 196) Then
        inputx = inputx - 150
    ElseIf (inputx = 32) Then
        inputx = 38
    ElseIf (inputx = 36) Then
        inputx = 39
    ElseIf (inputx = 37) Then
        inputx = 42
    ElseIf (inputx = 43) Then
        inputx = 41
    ElseIf (inputx = 45) Then
        inputx = 36
    ElseIf (inputx = 46) Then
        inputx = 37
    ElseIf (inputx = 47) Then
        inputx = 40
    ElseIf (inputx >= 48) And (inputx <= 57) Then
        inputx = inputx - 48
    ElseIf (inputx >= 65) And (inputx <= 90) Then
        inputx = inputx - 55
    End If

    getCode93Value = inputx

End Function

Public Function getCode93Character(ByVal inputx As Integer) As Integer

    If (inputx >= 43) And (inputx <= 46) Then
        inputx = inputx + 150
    ElseIf (inputx = 38) Then
        inputx = 32
    ElseIf (inputx = 39) Then
        inputx = 36
    ElseIf (inputx = 42) Then
        inputx = 37
    ElseIf (inputx = 41) Then
        inputx = 43
    ElseIf (inputx = 36) Then
        inputx = 45
    ElseIf (inputx = 37) Then
        inputx = 46
    ElseIf (inputx = 40) Then
        inputx = 47
    ElseIf (inputx >= 0) And (inputx <= 9) Then
        inputx = inputx + 48
    ElseIf (inputx >= 10) And (inputx <= 35) Then
        inputx = inputx + 55
    End If

    getCode93Character = inputx

End Function


Public Function generateCheckDigit_Code93(ByVal data As String) As String

    Dim cchk As Integer
    Dim kchk As Integer
    Dim sumx As Integer
    Dim x As Integer
    Dim translatedStr As String
    Dim weight As Integer
    Dim code93value As Integer
    Dim barcodechar As String
    Dim forceReturn As Integer
    

       
    forceReturn = 0
    sumx = 0
    x = 0
    translatedStr = ""
    weight = 1
    code93value = 0
    
    x = Len(data) - 1
    Do While (x >= 0)
    
        barcodechar = Mid(data, x + 1, 1)
        translatedStr = translate_Code93(Asc(barcodechar))
        If (Len(translatedStr) = 2) Then
        
            'code93value=getCode93Value((unsigned char) translatedStr[1]);
            code93value = getCode93Value(Asc(Mid(translatedStr, 1 + 1, 1)))
            
            'weight = (weight>20) ? 1 : weight
            If (weight > 20) Then
                weight = 1
            End If
            sumx = sumx + (weight * code93value)
            weight = weight + 1

            'code93value=getCode93Value((unsigned char) translatedStr[0]);
            code93value = getCode93Value(Asc(Mid(translatedStr, 0 + 1, 1)))
            If (weight > 20) Then
                weight = 1
            End If
            sumx = sumx + (weight * code93value)
            weight = weight + 1
        
        ElseIf (Len(translatedStr) = 1) Then
        
            'code93value=getCode93Value((unsigned char) translatedStr[0]);
            code93value = getCode93Value(Asc(Mid(translatedStr, 0 + 1, 1)))
            If (weight > 20) Then
                weight = 1
            End If
            sumx = sumx + (weight * code93value)
            weight = weight + 1
        
        Else
            forceReturn = 1  'return "";
        End If

        x = x - 1
    
    Loop

    cchk = sumx Mod 47
    weight = 2
    sumx = sumx Mod 47
    x = Len(data) - 1
    Do While (x >= 0)
    
        barcodechar = Mid(data, x + 1, 1)
        translatedStr = translate_Code93(Asc(barcodechar))
        
        If (Len(translatedStr) = 2) Then
        
            'code93value=getCode93Value((unsigned char) translatedStr[1]);
            code93value = getCode93Value(Asc(Mid(translatedStr, 1 + 1, 1)))
            If (weight > 15) Then
                weight = 1
            End If
            sumx = sumx + (weight * code93value)
            weight = weight + 1

            'code93value=getCode93Value((unsigned char) translatedStr[0]);
            code93value = getCode93Value(Asc(Mid(translatedStr, 0 + 1, 1)))
            If (weight > 15) Then
                weight = 1
            End If
            sumx = sumx + (weight * code93value)
            weight = weight + 1
        
        ElseIf (Len(translatedStr) = 1) Then
        
            'code93value=getCode93Value((unsigned char) translatedStr[0]);
            code93value = getCode93Value(Asc(Mid(translatedStr, 0 + 1, 1)))
            If (weight > 15) Then
                weight = 1
            End If
            sumx = sumx + (weight * code93value)
            weight = weight + 1
        Else
            forceReturn = 1  'return "";
        End If

        x = x - 1
    
    Loop
    kchk = sumx Mod 47
    
    If (forceReturn = 1) Then
        generateCheckDigit_Code93 = ""
    Else
        generateCheckDigit_Code93 = Chr(getCode93Character(cchk)) + Chr(getCode93Character(kchk))
    End If

End Function

' ==========================================================
' CodeCodabar
' ==========================================================
Public Function Encode_CodeCodabar(ByVal data As String) As String

    cd = ""
    result = ""
    filtereddata = filterInput_CodeCodabar(data)
    filteredlength = Len(filtereddata)

    If (filteredlength > 255) Then
        filtereddata = left$(filtereddata, 255)
    End If

    Encode_CodeCodabar = filtereddata

End Function


Public Function getCodeCodabarValue(ByVal inputchar As String) As Integer

  'Dim CODECODABARMAP(43) As String
  CODECODABARMAP = Array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "-", ".", "$", "/", "+", ":")
      
  returnVal = -1
  For x = 0 To 20 - 1
    
        If (CODECODABARMAP(x) = inputchar) Then
            returnVal = x
            Exit For
        End If
  Next x
  getCodeCodabarValue = returnVal
  
End Function

Public Function filterInput_CodeCodabar(ByVal data As String) As String

  result = ""
  datalength = Len(data)

  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    If (getCodeCodabarValue(barcodechar) <> -1) Then
        
        result = result + barcodechar
    End If
  
  Next x

  filterInput_CodeCodabar = result

End Function

'======================================================================================================
'POSTNET
'======================================================================================================
Public Function Encode_POSTNET(ByVal data As String) As String

    cd = ""
    result = ""
    filtereddata = filterInput_POSTNET(data)
    filteredlength = Len(filtereddata)

    If (filteredlength > 11) Then
        filtereddata = left$(filtereddata, 11)
    End If

    cd = generateCheckDigit_POSTNET(filtereddata)
    result = "{"+filtereddata + cd+"}"
    Encode_POSTNET = result

End Function



Public Function generateCheckDigit_POSTNET(ByVal data As String) As String

  datalength = 0
  sumx = 0
  result = -1
  strResult = ""
  barcodechar = ""

  datalength = Len(data)
  For x = 0 To datalength - 1
        
        barcodechar = Mid(data, x + 1, 1)
        sumx = sumx + getPOSTNETValue(Asc(barcodechar))
  Next x

  result = sumx Mod 10
  If (result <> 0) Then
    result = (10 - result)
  End If

  strResult = Chr(getPOSTNETCharacter(result))

  generateCheckDigit_POSTNET = strResult


End Function


Public Function getPOSTNETCharacter(ByVal inputdecimal As Integer) As Integer

    getPOSTNETCharacter = inputdecimal + 48

End Function


Public Function getPOSTNETValue(ByVal inputchar As Integer) As Integer

    getPOSTNETValue = inputchar - 48

End Function
    


Public Function filterInput_POSTNET(ByVal data As String) As String

  result = ""
  datalength = Len(data)

  For x = 0 To datalength - 1
  
    barcodechar = Mid(data, x + 1, 1)
    If (Asc(barcodechar) >= Asc("0") And Asc(barcodechar) <= Asc("9")) Then
         result = result + barcodechar
    End If
  Next x

 filterInput_POSTNET = result


End Function

'======================================================================================================
'Helper Functions for GS1 Databar
'======================================================================================================

Function filterInput_GS1Databar14(ByVal data As String) As String

    Dim datalength As Long

    result = ""
    datalength = Len(data)
    
    
    barcodechar = ""
    For x = 0 To datalength - 1
  
        barcodechar = Mid(data, x + 1, 1)
        If AscW(barcodechar) <= AscW("9") And AscW(barcodechar) >= AscW("0") Then
             result = result + barcodechar
        End If
    Next x
  
    filterInput_GS1Databar14 = result

End Function

Function GS1_MOD_10(ByVal data As String) As String

    Dim result As String
    Dim multi As Integer
    Dim total As Integer
    Dim cv As Integer
    
    If (Len(data) = 0) Then
        result = ""
    Else
    cv = 0
    multi = 3
    total = 0
    result = ""
    
    For x = 1 To Len(data)
        If (Asc(Mid(data, x, 1)) - 48 <= 9 And Asc(Mid(data, x, 1)) - 48 >= 0) Then
            result = result + Mid(data, x, 1)
        End If
    Next x

    For x = 1 To Len(result)
        cv = Asc(Mid(result, x, 1)) - 48
        total = total + cv * multi
        If (multi = 3) Then
            multi = 1
        Else
            multi = 3
        End If
            
    Next x
    
    Dim modresult As Integer
    modresult = total Mod 10
    If (modresult <> 0) Then
        modresult = 10 - modresult
    End If
    
    result = CStr(modresult)
    End If
    
    GS1_MOD_10 = result

End Function


Public Function combins__GS1Databar14(ByVal n As Long, ByVal r As Long) As Long

    Dim i, j As Long
    Dim maxDenom, minDenom As Long
    Dim val As Long
    
        If (n - r > r) Then
        
            minDenom = r
            maxDenom = n - r
        
        Else
            minDenom = n - r
            maxDenom = r
        End If
        
        
        val = 1
        j = 1
        For i = n To maxDenom + 1 Step -1
        
            val = val * i
            If (j <= minDenom) Then
                val = val / j
                j = j + 1
            End If
            
        Next i
        Do While (j <= minDenom)
            
            val = val / j
            j = j + 1
            
        Loop
            
        combins__GS1Databar14 = val

End Function

Public Function getGS1widths(ByVal val As Long, ByVal n As Long, ByVal elements As Long, ByVal maxWidth As Long, ByVal noNarrow As Long, ByRef widths() As Long) As Long

    Dim bar As Long
    Dim elmWidth As Long
    Dim i As Long
    Dim mxwElement As Long
    Dim subVal, lessVal As Long
    Dim narrowMask As Long
    Dim tempMask As Long
    
           
    
    narrowMask = 0
    For bar = 0 To (elements - 1 - 1)
    
        'for (elmWidth = 1, narrowMask |= (1<<bar);; elmWidth++, narrowMask &= ~(1<<bar))
        elmWidth = 1
        tempMask = 2 ^ bar
        narrowMask = narrowMask Or tempMask
            
        Do While (1 > 0)
        
                    ' get all combinations
                    subVal = combins__GS1Databar14(n - elmWidth - 1, elements - bar - 2)
                    ' less combinations with no single module element
                    'if ((!noNarrow) and (narrowMask==0) and
                    If ((noNarrow = 0) And (narrowMask = 0) And (n - elmWidth - (elements - bar - 1) >= elements - bar - 1)) Then
                        subVal = subVal - combins__GS1Databar14(n - elmWidth - (elements - bar), elements - bar - 2)
                    End If
                    ' less combinations with elements > maxval
                    If (elements - bar - 1 > 1) Then
                    
                        'for (mxwElement = n-elmWidth-(elements-bar-2);  mxwElement > maxWidth; mxwElement--)
                        lessVal = 0
                        mxwElement = n - elmWidth - (elements - bar - 2)
                        Do While (mxwElement > maxWidth)
                                                
                            lessVal = lessVal + combins__GS1Databar14(n - elmWidth - mxwElement - 1, elements - bar - 3)
                            mxwElement = mxwElement - 1
                        
                        Loop
                        subVal = subVal - (lessVal * (elements - 1 - bar))
                                            
                    ElseIf (n - elmWidth > maxWidth) Then
                    
                        subVal = subVal - 1
                        
                    End If
                    val = val - subVal
                    If (val < 0) Then
                        Exit Do
                    End If
                    
                    
                    elmWidth = elmWidth + 1
                    tempMask = 2 ^ bar
                    narrowMask = narrowMask And (Not (tempMask))
        Loop
            
        val = val + subVal
        n = n - elmWidth
        widths(bar) = elmWidth
        
    
    Next bar
    widths(bar) = n
    getGS1widths = 0

End Function

'======================================================================================================
'GS1 Databar-14
'======================================================================================================
Public Function Encode_GS1Databar14(ByVal data As String, Optional ByVal linkage As Integer = 0) As String

    Dim result As String
    Dim cd As String
    Dim filtereddata As String
    Dim addcharlength As Long
    Dim humanText As String
    Dim x As Long
    Dim leftstr, rightstr As String
    Dim valuez, leftz, rightz As Currency
    Dim tempasc As Long
    Dim left, right, data1, data2, data3, data4, sum, cleft, cright, ctemp, checksumval As Long
    Dim widths() As Variant
    Dim tempChar As String
    Dim bwresult As String
    Dim black As Long
    Dim retval As Long
    
    Dim checksum(4, 8) As Long
    Dim GS1DATABAR14FINDERS(9, 5) As Long
    
    Dim widthsodd(4) As Long
    Dim widthseven(4) As Long
    Dim results1odd(4) As Long
    Dim results2odd(4) As Long
    Dim results3odd(4) As Long
    Dim results4odd(4) As Long
    
    Dim results1even(4) As Long
    Dim results2even(4) As Long
    Dim results3even(4) As Long
    Dim results4even(4) As Long
    
    'CHECKSUM
    checksum(0, 0) = 1
    checksum(0, 1) = 3
    checksum(0, 2) = 9
    checksum(0, 3) = 27
    checksum(0, 4) = 2
    checksum(0, 5) = 6
    checksum(0, 6) = 18
    checksum(0, 7) = 54
    
    checksum(1, 0) = 4
    checksum(1, 1) = 12
    checksum(1, 2) = 36
    checksum(1, 3) = 29
    checksum(1, 4) = 8
    checksum(1, 5) = 24
    checksum(1, 6) = 72
    checksum(1, 7) = 58
    
    checksum(2, 0) = 16
    checksum(2, 1) = 48
    checksum(2, 2) = 65
    checksum(2, 3) = 37
    checksum(2, 4) = 32
    checksum(2, 5) = 17
    checksum(2, 6) = 51
    checksum(2, 7) = 74
    
    checksum(3, 0) = 64
    checksum(3, 1) = 34
    checksum(3, 2) = 23
    checksum(3, 3) = 69
    checksum(3, 4) = 49
    checksum(3, 5) = 68
    checksum(3, 6) = 46
    checksum(3, 7) = 59
    
    
    'GS1DATABAR14FINDERS
    GS1DATABAR14FINDERS(0, 0) = 3
    GS1DATABAR14FINDERS(0, 1) = 8
    GS1DATABAR14FINDERS(0, 2) = 2
    GS1DATABAR14FINDERS(0, 3) = 1
    GS1DATABAR14FINDERS(0, 4) = 1
    
    GS1DATABAR14FINDERS(1, 0) = 3
    GS1DATABAR14FINDERS(1, 1) = 5
    GS1DATABAR14FINDERS(1, 2) = 5
    GS1DATABAR14FINDERS(1, 3) = 1
    GS1DATABAR14FINDERS(1, 4) = 1
    
    GS1DATABAR14FINDERS(2, 0) = 3
    GS1DATABAR14FINDERS(2, 1) = 3
    GS1DATABAR14FINDERS(2, 2) = 7
    GS1DATABAR14FINDERS(2, 3) = 1
    GS1DATABAR14FINDERS(2, 4) = 1
        
    GS1DATABAR14FINDERS(3, 0) = 3
    GS1DATABAR14FINDERS(3, 1) = 1
    GS1DATABAR14FINDERS(3, 2) = 9
    GS1DATABAR14FINDERS(3, 3) = 1
    GS1DATABAR14FINDERS(3, 4) = 1
    
    GS1DATABAR14FINDERS(4, 0) = 2
    GS1DATABAR14FINDERS(4, 1) = 7
    GS1DATABAR14FINDERS(4, 2) = 4
    GS1DATABAR14FINDERS(4, 3) = 1
    GS1DATABAR14FINDERS(4, 4) = 1
    
    GS1DATABAR14FINDERS(5, 0) = 2
    GS1DATABAR14FINDERS(5, 1) = 5
    GS1DATABAR14FINDERS(5, 2) = 6
    GS1DATABAR14FINDERS(5, 3) = 1
    GS1DATABAR14FINDERS(5, 4) = 1
    
    GS1DATABAR14FINDERS(6, 0) = 2
    GS1DATABAR14FINDERS(6, 1) = 3
    GS1DATABAR14FINDERS(6, 2) = 8
    GS1DATABAR14FINDERS(6, 3) = 1
    GS1DATABAR14FINDERS(6, 4) = 1
    
    GS1DATABAR14FINDERS(7, 0) = 1
    GS1DATABAR14FINDERS(7, 1) = 5
    GS1DATABAR14FINDERS(7, 2) = 7
    GS1DATABAR14FINDERS(7, 3) = 1
    GS1DATABAR14FINDERS(7, 4) = 1
    
    GS1DATABAR14FINDERS(8, 0) = 1
    GS1DATABAR14FINDERS(8, 1) = 3
    GS1DATABAR14FINDERS(8, 2) = 9
    GS1DATABAR14FINDERS(8, 3) = 1
    GS1DATABAR14FINDERS(8, 4) = 1
    
             
    
    cd = ""
    result = ""
    filtereddata = filterInput_GS1Databar14(data)
    filteredlength = Len(filtereddata)

    If filteredlength > 14 Then
            filtereddata = VBA.left$(filtereddata, 14)
    End If
     
    If (filteredlength < 14) Then
        addcharlength = 14 - filteredlength -1
        For x = 0 To addcharlength - 1
            filtereddata = "0" + filtereddata
        Next x
	filtereddata=filtereddata+GS1_MOD_10(filtereddata)

    End If
    
       
    humanText = "(01)" + VBA.right$(filtereddata, 14)
    filtereddata = VBA.left$(filtereddata, 13) 'Exclude Check Digit
    
    'ver 1.1
    If (linkage = 1) Then
        filtereddata = "1" + filtereddata
    End If
    
    filteredlength = VBA.Len(filtereddata)
    
    valuez = 0
    For x = 0 To (filteredlength - 1)
        tempChar = Mid(filtereddata, x + 1, 1)
        tempasc = AscW(tempChar)
        valuez = tempasc - 48 + (valuez * 10)
    Next x
    
    
    sum = 0
    
    leftz = valuez / 4537077
    
    'Check
    left = Int(leftz)
    rightz = valuez - left * 4537077
    right = rightz
    
    data1 = Int(left / 1597)
    data2 = left Mod 1597
    data3 = Int(right / 1597)
    data4 = right Mod 1597
    
    'MsgBox left
    'MsgBox right
    'MsgBox data1
    'MsgBox data2
    'MsgBox data3
    'MsgBox data4
    
    retval = getGS1W(data1, 1, 16, widthsodd)
    retval = getGS1W(data1, 0, 16, widthseven)
  
    For x = 0 To 3
        
        
        results1odd(x) = widthsodd(x)
        results1even(x) = widthseven(x)
        sum = sum + checksum(0, (x * 2)) * widthsodd(x) + checksum(0, (x * 2) + 1) * widthseven(x)
        'teststr = teststr + Str(results1odd(x)) + "," + Str(results1even(x)) + ","
    
    Next x
    
    
    retval = getGS1W(data2, 1, 15, widthsodd)
    retval = getGS1W(data2, 0, 15, widthseven)
    For x = 0 To 3
        
        results2odd(x) = widthsodd(x)
        results2even(x) = widthseven(x)
        sum = sum + checksum(1, (x * 2)) * widthsodd(x) + checksum(1, (x * 2) + 1) * widthseven(x)
        'teststr = teststr + Str(results2odd(x)) + "," + Str(results2even(x)) + ","
        
    Next x
    
    
    retval = getGS1W(data3, 1, 16, widthsodd)
    retval = getGS1W(data3, 0, 16, widthseven)
    For x = 0 To 3
        
        
        results3odd(x) = widthsodd(x)
        results3even(x) = widthseven(x)
        sum = sum + checksum(2, (x * 2)) * widthsodd(x) + checksum(2, (x * 2) + 1) * widthseven(x)
        'teststr = teststr + Str(results3odd(x)) + "," + Str(results3even(x)) + ","
    Next x
    
        
    retval = getGS1W(data4, 1, 15, widthsodd)
    retval = getGS1W(data4, 0, 15, widthseven)
    For x = 0 To 3
        
        results4odd(x) = widthsodd(x)
        results4even(x) = widthseven(x)
        sum = sum + checksum(3, (x * 2)) * widthsodd(x) + checksum(3, (x * 2) + 1) * widthseven(x)
        'teststr = teststr + Str(results4odd(x)) + "," + Str(results4even(x)) + ","
        
    Next x
    
    
        
    checksumval = sum Mod 79
    
    '1031
    ctemp = checksumval
    If (ctemp >= 8) Then
        ctemp = ctemp + 1
    End If
        
    If (ctemp >= 72) Then
        ctemp = ctemp + 1
    End If


    cleft = Int(ctemp / 9)
    
    'MsgBox cleft
    cright = ctemp Mod 9
    
    
    result = "11" 'Left Guard
    
    For x = 0 To 3 'Data 1
        result = result + ChrW(results1odd(x) + 48) + ChrW(results1even(x) + 48)
    Next x
    
    For x = 0 To 4 'Left Check
        result = result + ChrW(GS1DATABAR14FINDERS(cleft, x) + 48)
    Next x
    
    For x = 3 To 0 Step -1 'Data 2
        result = result + ChrW(results2even(x) + 48) + ChrW(results2odd(x) + 48)
    Next x
    
    For x = 0 To 3 'Data 4
        result = result + ChrW(results4odd(x) + 48) + ChrW(results4even(x) + 48)
    Next x
        
    For x = 4 To 0 Step -1 'Right Check
        result = result + ChrW(GS1DATABAR14FINDERS(cright, x) + 48)
    Next x
    
    For x = 3 To 0 Step -1 'Data 3
        result = result + ChrW(results3even(x) + 48) + ChrW(results3odd(x) + 48)
    Next x
    result = result + "11" 'Right Guard
    
    
    
    'Convert to White Black White Black
    bwresult = ""
    black = 0
    For x = 0 To Len(result) - 1
    
        If (black = 0) Then
        
            bwresult = bwresult + ChrW(AscW(Mid(result, x + 1, 1)) + 48)
            black = 1
        
        Else
        
            bwresult = bwresult + ChrW(AscW(Mid(result, x + 1, 1)) + 16)
            black = 0
        
        End If

    
    Next x
    
    Encode_GS1Databar14 = bwresult

End Function


Public Function getGS1W(ByVal data As Long, ByVal oddeven As Long, ByVal modules As Long, ByRef widths() As Long) As Long

    Dim retval As Long
    Dim WIDTH16_4_0(5, 7) As Long
    Dim WIDTH16_4_1(5, 7) As Long
    Dim WIDTH15_4_0(4, 7) As Long
    Dim WIDTH15_4_1(4, 7) As Long
        
    '640
    WIDTH16_4_0(0, 0) = 0
    WIDTH16_4_0(0, 1) = 160
    WIDTH16_4_0(0, 2) = 1
    WIDTH16_4_0(0, 3) = 4
    WIDTH16_4_0(0, 4) = 4
    WIDTH16_4_0(0, 5) = 1
    WIDTH16_4_0(0, 6) = 0

    WIDTH16_4_0(1, 0) = 161
    WIDTH16_4_0(1, 1) = 960
    WIDTH16_4_0(1, 2) = 10
    WIDTH16_4_0(1, 3) = 6
    WIDTH16_4_0(1, 4) = 4
    WIDTH16_4_0(1, 5) = 3
    WIDTH16_4_0(1, 6) = 0
    
    WIDTH16_4_0(2, 0) = 961
    WIDTH16_4_0(2, 1) = 2014
    WIDTH16_4_0(2, 2) = 34
    WIDTH16_4_0(2, 3) = 8
    WIDTH16_4_0(2, 4) = 4
    WIDTH16_4_0(2, 5) = 5
    WIDTH16_4_0(2, 6) = 0
    
    WIDTH16_4_0(3, 0) = 2015
    WIDTH16_4_0(3, 1) = 2714
    WIDTH16_4_0(3, 2) = 70
    WIDTH16_4_0(3, 3) = 10
    WIDTH16_4_0(3, 4) = 4
    WIDTH16_4_0(3, 5) = 6
    WIDTH16_4_0(3, 6) = 0

    WIDTH16_4_0(4, 0) = 2715
    WIDTH16_4_0(4, 1) = 2840
    WIDTH16_4_0(4, 2) = 126
    WIDTH16_4_0(4, 3) = 12
    WIDTH16_4_0(4, 4) = 4
    WIDTH16_4_0(4, 5) = 8
    WIDTH16_4_0(4, 6) = 0
        

               
    '641
    WIDTH16_4_1(0, 0) = 0
    WIDTH16_4_1(0, 1) = 160
    WIDTH16_4_1(0, 2) = 1
    WIDTH16_4_1(0, 3) = 12
    WIDTH16_4_1(0, 4) = 4
    WIDTH16_4_1(0, 5) = 8
    WIDTH16_4_1(0, 6) = 1

    WIDTH16_4_1(1, 0) = 161
    WIDTH16_4_1(1, 1) = 960
    WIDTH16_4_1(1, 2) = 10
    WIDTH16_4_1(1, 3) = 10
    WIDTH16_4_1(1, 4) = 4
    WIDTH16_4_1(1, 5) = 6
    WIDTH16_4_1(1, 6) = 1

    WIDTH16_4_1(2, 0) = 961
    WIDTH16_4_1(2, 1) = 2014
    WIDTH16_4_1(2, 2) = 34
    WIDTH16_4_1(2, 3) = 8
    WIDTH16_4_1(2, 4) = 4
    WIDTH16_4_1(2, 5) = 4
    WIDTH16_4_1(2, 6) = 1

    WIDTH16_4_1(3, 0) = 2015
    WIDTH16_4_1(3, 1) = 2714
    WIDTH16_4_1(3, 2) = 70
    WIDTH16_4_1(3, 3) = 6
    WIDTH16_4_1(3, 4) = 4
    WIDTH16_4_1(3, 5) = 3
    WIDTH16_4_1(3, 6) = 1

    WIDTH16_4_1(4, 0) = 2715
    WIDTH16_4_1(4, 1) = 2840
    WIDTH16_4_1(4, 2) = 126
    WIDTH16_4_1(4, 3) = 4
    WIDTH16_4_1(4, 4) = 4
    WIDTH16_4_1(4, 5) = 1
    WIDTH16_4_1(4, 6) = 1
    
        
    '540
    WIDTH15_4_0(0, 0) = 0
    WIDTH15_4_0(0, 1) = 335
    WIDTH15_4_0(0, 2) = 4
    WIDTH15_4_0(0, 3) = 10
    WIDTH15_4_0(0, 4) = 4
    WIDTH15_4_0(0, 5) = 7
    WIDTH15_4_0(0, 6) = 1
    
    WIDTH15_4_0(1, 0) = 336
    WIDTH15_4_0(1, 1) = 1035
    WIDTH15_4_0(1, 2) = 20
    WIDTH15_4_0(1, 3) = 8
    WIDTH15_4_0(1, 4) = 4
    WIDTH15_4_0(1, 5) = 5
    WIDTH15_4_0(1, 6) = 1
    
    WIDTH15_4_0(2, 0) = 1036
    WIDTH15_4_0(2, 1) = 1515
    WIDTH15_4_0(2, 2) = 48
    WIDTH15_4_0(2, 3) = 6
    WIDTH15_4_0(2, 4) = 4
    WIDTH15_4_0(2, 5) = 3
    WIDTH15_4_0(2, 6) = 1
        
    WIDTH15_4_0(3, 0) = 1516
    WIDTH15_4_0(3, 1) = 1596
    WIDTH15_4_0(3, 2) = 81
    WIDTH15_4_0(3, 3) = 4
    WIDTH15_4_0(3, 4) = 4
    WIDTH15_4_0(3, 5) = 1
    WIDTH15_4_0(3, 6) = 1
    
    
    '541
    WIDTH15_4_1(0, 0) = 0
    WIDTH15_4_1(0, 1) = 335
    WIDTH15_4_1(0, 2) = 4
    WIDTH15_4_1(0, 3) = 5
    WIDTH15_4_1(0, 4) = 4
    WIDTH15_4_1(0, 5) = 2
    WIDTH15_4_1(0, 6) = 0
     
    WIDTH15_4_1(1, 0) = 336
    WIDTH15_4_1(1, 1) = 1035
    WIDTH15_4_1(1, 2) = 20
    WIDTH15_4_1(1, 3) = 7
    WIDTH15_4_1(1, 4) = 4
    WIDTH15_4_1(1, 5) = 4
    WIDTH15_4_1(1, 6) = 0
    
    WIDTH15_4_1(2, 0) = 1036
    WIDTH15_4_1(2, 1) = 1515
    WIDTH15_4_1(2, 2) = 48
    WIDTH15_4_1(2, 3) = 9
    WIDTH15_4_1(2, 4) = 4
    WIDTH15_4_1(2, 5) = 6
    WIDTH15_4_1(2, 6) = 0
    
    WIDTH15_4_1(3, 0) = 1516
    WIDTH15_4_1(3, 1) = 1596
    WIDTH15_4_1(3, 2) = 81
    WIDTH15_4_1(3, 3) = 11
    WIDTH15_4_1(3, 4) = 4
    WIDTH15_4_1(3, 5) = 8
    WIDTH15_4_1(3, 6) = 0

                        
    If (modules = 16) Then
    
         If (oddeven = 0) Then
        
            'for (int x=0;x<5;x++)
            For x = 0 To 4
                If (data >= WIDTH16_4_0(x, 0) And data <= WIDTH16_4_0(x, 1)) Then
                    'getGS1widths(((data-WIDTH16_4_0(x,0)) mod WIDTH16_4_0(x,2)),WIDTH16_4_0(x,3),WIDTH16_4_0(x,4),WIDTH16_4_0(x,5),WIDTH16_4_0(x,6))
                    retval = getGS1widths(Int((data - WIDTH16_4_0(x, 0)) Mod WIDTH16_4_0(x, 2)), WIDTH16_4_0(x, 3), WIDTH16_4_0(x, 4), WIDTH16_4_0(x, 5), WIDTH16_4_0(x, 6), widths)
                
                End If
            Next x
        
        Else
            
            
            For x = 0 To 4
                
                If (data >= WIDTH16_4_1(x, 0) And data <= WIDTH16_4_1(x, 1)) Then
                    'teststr = Str(data) + " " + Str(WIDTH16_4_1(x, 0)) + " " + Str(WIDTH16_4_1(x, 2)) + Str(((data - WIDTH16_4_1(x, 0)) / WIDTH16_4_1(x, 2)))
                    retval = getGS1widths(Int((data - WIDTH16_4_1(x, 0)) / WIDTH16_4_1(x, 2)), WIDTH16_4_1(x, 3), WIDTH16_4_1(x, 4), WIDTH16_4_1(x, 5), WIDTH16_4_1(x, 6), widths)
                End If
                
             Next x
             
        End If
    
    ElseIf (modules = 15) Then
    
        If (oddeven = 0) Then
        
            
            For x = 0 To 3
                If (data >= WIDTH15_4_0(x, 0) And data <= WIDTH15_4_0(x, 1)) Then
                    'teststr = Str(data) + " " + Str(WIDTH15_4_0(x, 0)) + " " + Str(WIDTH15_4_0(x, 2)) + Str((data - WIDTH15_4_0(x, 0)) / WIDTH15_4_0(x, 2))
                    retval = getGS1widths(Int((data - WIDTH15_4_0(x, 0)) / WIDTH15_4_0(x, 2)), WIDTH15_4_0(x, 3), WIDTH15_4_0(x, 4), WIDTH15_4_0(x, 5), WIDTH15_4_0(x, 6), widths)
                End If
            Next x
            
        
        Else
        
            
            For x = 0 To 3
                If (data >= WIDTH15_4_1(x, 0) And data <= WIDTH15_4_1(x, 1)) Then
                    retval = getGS1widths(Int((data - WIDTH15_4_1(x, 0)) Mod WIDTH15_4_1(x, 2)), WIDTH15_4_1(x, 3), WIDTH15_4_1(x, 4), WIDTH15_4_1(x, 5), WIDTH15_4_1(x, 6), widths)
                End If
            Next x
        
        End If
        
    End If
    
    getGS1W = 0

End Function

